# coding: utf-8
# /*##########################################################################
# Copyright (C) 2016-2018 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ############################################################################*/
"""Tests for SpecFile to HDF5 converter"""

from numpy import array_equal
import os
import sys
import tempfile
import unittest

try:
    import h5py
except ImportError:
    h5py_missing = True
else:
    h5py_missing = False
    from ..spech5 import SpecH5, SpecH5Group
    from ..convert import convert, write_to_h5

__authors__ = ["P. Knobel"]
__license__ = "MIT"
__date__ = "12/02/2018"


sftext = """#F /tmp/sf.dat
#E 1455180875
#D Thu Feb 11 09:54:35 2016
#C imaging  User = opid17
#O0 Pslit HGap  MRTSlit UP  MRTSlit DOWN
#O1 Sslit1 VOff  Sslit1 HOff  Sslit1 VGap
#o0 pshg mrtu mrtd
#o2 ss1vo ss1ho ss1vg

#J0 Seconds  IA  ion.mono  Current
#J1 xbpmc2  idgap1  Inorm

#S 1  ascan  ss1vo -4.55687 -0.556875  40 0.2
#D Thu Feb 11 09:55:20 2016
#T 0.2  (Seconds)
#P0 180.005 -0.66875 0.87125
#P1 14.74255 16.197579 12.238283
#N 4
#L MRTSlit UP  second column  3rd_col
-1.23 5.89  8
8.478100E+01  5 1.56
3.14 2.73 -3.14
1.2 2.3 3.4

#S 1 aaaaaa
#D Thu Feb 11 10:00:32 2016
#@MCADEV 1
#@MCA %16C
#@CHANN 3 0 2 1
#@CALIB 1 2 3
#N 3
#L uno  duo
1 2
@A 0 1 2
@A 10 9 8
3 4
@A 3.1 4 5
@A 7 6 5
5 6
@A 6 7.7 8
@A 4 3 2
"""


@unittest.skipIf(h5py_missing, "Could not import h5py")
class TestConvertSpecHDF5(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        fd, cls.spec_fname = tempfile.mkstemp(text=False)
        if sys.version_info < (3, ):
            os.write(fd, sftext)
        else:
            os.write(fd, bytes(sftext, 'ascii'))
        os.close(fd)

        fd, cls.h5_fname = tempfile.mkstemp(text=False)
        # Close and delete (we just need the name)
        os.close(fd)
        os.unlink(cls.h5_fname)

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls.spec_fname)

    def setUp(self):
        convert(self.spec_fname, self.h5_fname)

        self.sfh5 = SpecH5(self.spec_fname)
        self.h5f = h5py.File(self.h5_fname, "a")

    def tearDown(self):
        self.h5f.close()
        self.sfh5.close()
        os.unlink(self.h5_fname)

    def testAppendToHDF5(self):
        write_to_h5(self.sfh5, self.h5f, h5path="/foo/bar/spam")
        self.assertTrue(
            array_equal(self.h5f["/1.2/measurement/mca_1/data"],
                        self.h5f["/foo/bar/spam/1.2/measurement/mca_1/data"])
        )

    def testWriteSpecH5Group(self):
        """Test passing a SpecH5Group as parameter, instead of a Spec filename
        or a SpecH5."""
        g = self.sfh5["1.1/instrument"]
        self.assertIsInstance(g, SpecH5Group)        # let's be paranoid
        write_to_h5(g, self.h5f, h5path="my instruments")

        self.assertAlmostEqual(self.h5f["my instruments/positioners/Sslit1 HOff"][tuple()],
                               16.197579, places=4)

    def testTitle(self):
        """Test the value of a dataset"""
        title12 = self.h5f["/1.2/title"].value
        self.assertEqual(title12,
                         u"aaaaaa")

    def testAttrs(self):
        # Test root group (file) attributes
        self.assertEqual(self.h5f.attrs["NX_class"],
                         u"NXroot")
        # Test dataset attributes
        ds = self.h5f["/1.2/instrument/mca_1/data"]
        self.assertTrue("interpretation" in ds.attrs)
        self.assertEqual(list(ds.attrs.values()),
                         [u"spectrum"])
        # Test group attributes
        grp = self.h5f["1.1"]
        self.assertEqual(grp.attrs["NX_class"],
                         u"NXentry")
        self.assertEqual(len(list(grp.attrs.keys())),
                         1)

    def testHdf5HasSameMembers(self):
        spec_member_list = []

        def append_spec_members(name):
            spec_member_list.append(name)
        self.sfh5.visit(append_spec_members)

        hdf5_member_list = []

        def append_hdf5_members(name):
            hdf5_member_list.append(name)
        self.h5f.visit(append_hdf5_members)

        # 1. For some reason, h5py visit method doesn't include the leading
        # "/" character when it passes the member name to the function,
        # even though an explicit the .name attribute of a member will
        # have a leading "/"
        spec_member_list = [m.lstrip("/") for m in spec_member_list]

        self.assertEqual(set(hdf5_member_list),
                         set(spec_member_list))

    def testLinks(self):
        self.assertTrue(
            array_equal(self.sfh5["/1.2/measurement/mca_0/data"],
                        self.h5f["/1.2/measurement/mca_0/data"])
        )
        self.assertTrue(
            array_equal(self.h5f["/1.2/instrument/mca_1/channels"],
                        self.h5f["/1.2/measurement/mca_1/info/channels"])
        )


def suite():
    test_suite = unittest.TestSuite()
    test_suite.addTest(
        unittest.defaultTestLoader.loadTestsFromTestCase(TestConvertSpecHDF5))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Tests for html module"""

__authors__ = ["V. Valls"]
__license__ = "MIT"
__date__ = "19/09/2016"


import unittest
from .. import html


class TestHtml(unittest.TestCase):
    """Tests for html module."""

    def testLtGt(self):
        result = html.escape("<html>'\"")
        self.assertEquals("&lt;html&gt;&apos;&quot;", result)

    def testLtAmpGt(self):
        # '&' have to be escaped first
        result = html.escape("<&>")
        self.assertEquals("&lt;&amp;&gt;", result)

    def testNoQuotes(self):
        result = html.escape("\"m&m's\"", quote=False)
        self.assertEquals("\"m&amp;m's\"", result)


def suite():
    test_suite = unittest.TestSuite()
    test_suite.addTest(
        unittest.defaultTestLoader.loadTestsFromTestCase(TestHtml))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""This module provides toolbars with tools for a Plot3DWidget.

It provides the following toolbars:

- :class:`InteractiveModeToolBar` with:
  - Set interactive mode to rotation
  - Set interactive mode to pan

- :class:`OutputToolBar` with:
  - Copy
  - Save
  - Video
  - Print
"""

from __future__ import absolute_import

__authors__ = ["T. Vincent"]
__license__ = "MIT"
__date__ = "06/09/2017"

import logging
import weakref

from silx.gui import qt

from .ViewpointTools import ViewpointToolButton
from .. import actions

_logger = logging.getLogger(__name__)


class Plot3DWidgetToolBar(qt.QToolBar):
    """Base class for toolbar associated to a Plot3DWidget

    :param parent: See :class:`QWidget`
    :param str title: Title of the toolbar.
    """

    def __init__(self, parent=None, title=''):
        super(Plot3DWidgetToolBar, self).__init__(title, parent)

        self._plot3DRef = None

    def _plot3DWidgetChanged(self, widget):
        """Handle change of Plot3DWidget and sync actions

        :param Plot3DWidget widget:
        """
        for action in self.actions():
            if isinstance(action, actions.Plot3DAction):
                action.setPlot3DWidget(widget)

    def setPlot3DWidget(self, widget):
        """Set the Plot3DWidget this toolbar is associated with

        :param Plot3DWidget widget: The widget to control
        """
        self._plot3DRef = None if widget is None else weakref.ref(widget)
        self._plot3DWidgetChanged(widget)

    def getPlot3DWidget(self):
        """Return the Plot3DWidget associated to this toolbar.

        If no widget is associated, it returns None.

        :rtype: qt.QWidget
        """
        return None if self._plot3DRef is None else self._plot3DRef()


class InteractiveModeToolBar(Plot3DWidgetToolBar):
    """Toolbar providing icons to change the interaction mode

    :param parent: See :class:`QWidget`
    :param str title: Title of the toolbar.
    """

    def __init__(self, parent=None, title='Plot3D Interaction'):
        super(InteractiveModeToolBar, self).__init__(parent, title)

        self._rotateAction = actions.mode.RotateArcballAction(parent=self)
        self.addAction(self._rotateAction)

        self._panAction = actions.mode.PanAction(parent=self)
        self.addAction(self._panAction)

    def getRotateAction(self):
        """Returns the QAction setting rotate interaction of the Plot3DWidget

        :rtype: qt.QAction
        """
        return self._rotateAction

    def getPanAction(self):
        """Returns the QAction setting pan interaction of the Plot3DWidget

        :rtype: qt.QAction
        """
        return self._panAction


class OutputToolBar(Plot3DWidgetToolBar):
    """Toolbar providing icons to copy, save and print the OpenGL scene

    :param parent: See :class:`QWidget`
    :param str title: Title of the toolbar.
    """

    def __init__(self, parent=None, title='Plot3D Output'):
        super(OutputToolBar, self).__init__(parent, title)

        self._copyAction = actions.io.CopyAction(parent=self)
        self.addAction(self._copyAction)

        self._saveAction = actions.io.SaveAction(parent=self)
        self.addAction(self._saveAction)

        self._videoAction = actions.io.VideoAction(parent=self)
        self.addAction(self._videoAction)

        self._printAction = actions.io.PrintAction(parent=self)
        self.addAction(self._printAction)

    def getCopyAction(self):
        """Returns the QAction performing copy to clipboard of the Plot3DWidget

        :rtype: qt.QAction
        """
        return self._copyAction

    def getSaveAction(self):
        """Returns the QAction performing save to file of the Plot3DWidget

        :rtype: qt.QAction
        """
        return self._saveAction

    def getVideoRecordAction(self):
        """Returns the QAction performing record video of the Plot3DWidget

        :rtype: qt.QAction
        """
        return self._videoAction

    def getPrintAction(self):
        """Returns the QAction performing printing of the Plot3DWidget

        :rtype: qt.QAction
        """
        return self._printAction


class ViewpointToolBar(Plot3DWidgetToolBar):
    """A toolbar providing icons to reset the viewpoint.

    :param parent: See :class:`QToolBar`
    :param str title: Title of the toolbar
    """

    def __init__(self, parent=None, title='Viewpoint control'):
        super(ViewpointToolBar, self).__init__(parent, title)

        self._viewpointToolButton = ViewpointToolButton(parent=self)
        self.addWidget(self._viewpointToolButton)
        self._rotateViewpointAction = actions.viewpoint.RotateViewpoint(parent=self)
        self.addAction(self._rotateViewpointAction)

    def _plot3DWidgetChanged(self, widget):
        self.getViewpointToolButton().setPlot3DWidget(widget)
        super(ViewpointToolBar, self)._plot3DWidgetChanged(widget)

    def getViewpointToolButton(self):
        """Returns the ViewpointToolButton to set viewpoint of the Plot3DWidget

        :rtype: ViewpointToolButton
        """
        return self._viewpointToolButton

    def getRotateViewpointAction(self):
        """Returns the QAction to start/stop rotation of the Plot3DWidget

        :rtype: qt.QAction
        """
        return self._rotateViewpointAction



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2014-2018 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ############################################################################*/
"""OpenGL Plot backend."""

from __future__ import division

__authors__ = ["T. Vincent"]
__license__ = "MIT"
__date__ = "24/04/2018"

from collections import OrderedDict, namedtuple
from ctypes import c_void_p
import logging

import numpy

from .._utils import FLOAT32_MINPOS
from . import BackendBase
from ... import colors
from ... import qt

from ..._glutils import gl
from ... import _glutils as glu
from .glutils import (
    GLPlotCurve2D, GLPlotColormap, GLPlotRGBAImage, GLPlotFrame2D,
    mat4Ortho, mat4Identity,
    LEFT, RIGHT, BOTTOM, TOP,
    Text2D, Shape2D)
from .glutils.PlotImageFile import saveImageToFile

_logger = logging.getLogger(__name__)


# TODO idea: BackendQtMixIn class to share code between mpl and gl
# TODO check if OpenGL is available
# TODO make an off-screen mesa backend

# Bounds ######################################################################

class Range(namedtuple('Range', ('min_', 'max_'))):
    """Describes a 1D range"""

    @property
    def range_(self):
        return self.max_ - self.min_

    @property
    def center(self):
        return 0.5 * (self.min_ + self.max_)


class Bounds(object):
    """Describes plot bounds with 2 y axis"""

    def __init__(self, xMin, xMax, yMin, yMax, y2Min, y2Max):
        self._xAxis = Range(xMin, xMax)
        self._yAxis = Range(yMin, yMax)
        self._y2Axis = Range(y2Min, y2Max)

    def __repr__(self):
        return "x: %s, y: %s, y2: %s" % (repr(self._xAxis),
                                         repr(self._yAxis),
                                         repr(self._y2Axis))

    @property
    def xAxis(self):
        return self._xAxis

    @property
    def yAxis(self):
        return self._yAxis

    @property
    def y2Axis(self):
        return self._y2Axis


# Content #####################################################################

class PlotDataContent(object):
    """Manage plot data content: images and curves.

    This class is only meant to work with _OpenGLPlotCanvas.
    """

    _PRIMITIVE_TYPES = 'curve', 'image'

    def __init__(self):
        self._primitives = OrderedDict()  # For images and curves

    def add(self, primitive):
        """Add a curve or image to the content dictionary.

        This function generates the key in the dict from the primitive.

        :param primitive: The primitive to add.
        :type primitive: Instance of GLPlotCurve2D, GLPlotColormap,
                         GLPlotRGBAImage.
        """
        if isinstance(primitive, GLPlotCurve2D):
            primitiveType = 'curve'
        elif isinstance(primitive, (GLPlotColormap, GLPlotRGBAImage)):
            primitiveType = 'image'
        else:
            raise RuntimeError('Unsupported object type: %s', primitive)

        key = primitiveType, primitive.info['legend']
        self._primitives[key] = primitive

    def get(self, primitiveType, legend):
        """Get the corresponding primitive of given type with given legend.

        :param str primitiveType: Type of primitive ('curve' or 'image').
        :param str legend: The legend of the primitive to retrieve.
        :return: The corresponding curve or None if no such curve.
        """
        assert primitiveType in self._PRIMITIVE_TYPES
        return self._primitives.get((primitiveType, legend))

    def pop(self, primitiveType, key):
        """Pop the corresponding curve or return None if no such curve.

        :param str primitiveType:
        :param str key:
        :return:
        """
        assert primitiveType in self._PRIMITIVE_TYPES
        return self._primitives.pop((primitiveType, key), None)

    def zOrderedPrimitives(self, reverse=False):
        """List of primitives sorted according to their z order.

        It is a stable sort (as sorted):
        Original order is preserved when key is the same.

        :param bool reverse: Ascending (True, default) or descending (False).
        """
        return sorted(self._primitives.values(),
                      key=lambda primitive: primitive.info['zOrder'],
                      reverse=reverse)

    def primitives(self):
        """Iterator over all primitives."""
        return self._primitives.values()

    def primitiveKeys(self, primitiveType):
        """Iterator over primitives of a specific type."""
        assert primitiveType in self._PRIMITIVE_TYPES
        for type_, key in self._primitives.keys():
            if type_ == primitiveType:
                yield key

    def getBounds(self, xPositive=False, yPositive=False):
        """Bounds of the data.

        Can return strictly positive bounds (for log scale).
        In this case, curves are clipped to their smaller positive value
        and images with negative min are ignored.

        :param bool xPositive: True to get strictly positive range.
        :param bool yPositive: True to get strictly positive range.
        :return: The range of data for x, y and y2, or default (1., 100.)
                 if no range found for one dimension.
        :rtype: Bounds
        """
        xMin, yMin, y2Min = float('inf'), float('inf'), float('inf')
        xMax = 0. if xPositive else -float('inf')
        if yPositive:
            yMax, y2Max = 0., 0.
        else:
            yMax, y2Max = -float('inf'), -float('inf')

        for item in self._primitives.values():
            # To support curve <= 0. and log and bypass images:
            # If positive only, uses x|yMinPos if available
            # and bypass other data with negative min bounds
            if xPositive:
                itemXMin = getattr(item, 'xMinPos', item.xMin)
                if itemXMin is None or itemXMin < FLOAT32_MINPOS:
                    continue
            else:
                itemXMin = item.xMin

            if yPositive:
                itemYMin = getattr(item, 'yMinPos', item.yMin)
                if itemYMin is None or itemYMin < FLOAT32_MINPOS:
                    continue
            else:
                itemYMin = item.yMin

            if itemXMin < xMin:
                xMin = itemXMin
            if item.xMax > xMax:
                xMax = item.xMax

            if item.info.get('yAxis') == 'right':
                if itemYMin < y2Min:
                    y2Min = itemYMin
                if item.yMax > y2Max:
                    y2Max = item.yMax
            else:
                if itemYMin < yMin:
                    yMin = itemYMin
                if item.yMax > yMax:
                    yMax = item.yMax

        # One of the limit has not been updated, return default range
        if xMin >= xMax:
            xMin, xMax = 1., 100.
        if yMin >= yMax:
            yMin, yMax = 1., 100.
        if y2Min >= y2Max:
            y2Min, y2Max = 1., 100.

        return Bounds(xMin, xMax, yMin, yMax, y2Min, y2Max)


# shaders #####################################################################

_baseVertShd = """
    attribute vec2 position;
    uniform mat4 matrix;
    uniform bvec2 isLog;

    const float oneOverLog10 = 0.43429448190325176;

    void main(void) {
        vec2 posTransformed = position;
        if (isLog.x) {
            posTransformed.x = oneOverLog10 * log(position.x);
        }
        if (isLog.y) {
            posTransformed.y = oneOverLog10 * log(position.y);
        }
        gl_Position = matrix * vec4(posTransformed, 0.0, 1.0);
    }
    """

_baseFragShd = """
    uniform vec4 color;
    uniform int hatchStep;
    uniform float tickLen;

    void main(void) {
        if (tickLen != 0.) {
            if (mod((gl_FragCoord.x + gl_FragCoord.y) / tickLen, 2.) < 1.) {
                gl_FragColor = color;
            } else {
                discard;
            }
        } else if (hatchStep == 0 ||
            mod(gl_FragCoord.x - gl_FragCoord.y, float(hatchStep)) == 0.) {
            gl_FragColor = color;
        } else {
            discard;
        }
    }
    """

_texVertShd = """
   attribute vec2 position;
   attribute vec2 texCoords;
   uniform mat4 matrix;

   varying vec2 coords;

   void main(void) {
        gl_Position = matrix * vec4(position, 0.0, 1.0);
        coords = texCoords;
   }
   """

_texFragShd = """
    uniform sampler2D tex;

    varying vec2 coords;

    void main(void) {
        gl_FragColor = texture2D(tex, coords);
        gl_FragColor.a = 1.0;
    }
    """


# BackendOpenGL ###############################################################

_current_context = None


def _getContext():
    assert _current_context is not None
    return _current_context


class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
    """OpenGL-based Plot backend.

    WARNINGS:
    Unless stated otherwise, this API is NOT thread-safe and MUST be
    called from the main thread.
    When numpy arrays are passed as arguments to the API (through
    :func:`addCurve` and :func:`addImage`), they are copied only if
    required.
    So, the caller should not modify these arrays afterwards.
    """

    _sigPostRedisplay = qt.Signal()
    """Signal handling automatic asynchronous replot"""

    def __init__(self, plot, parent=None, f=qt.Qt.WindowFlags()):
        glu.OpenGLWidget.__init__(self, parent,
                                  alphaBufferSize=8,
                                  depthBufferSize=0,
                                  stencilBufferSize=0,
                                  version=(2, 1),
                                  f=f)
        BackendBase.BackendBase.__init__(self, plot, parent)

        self.matScreenProj = mat4Identity()

        self._progBase = glu.Program(
            _baseVertShd, _baseFragShd, attrib0='position')
        self._progTex = glu.Program(
            _texVertShd, _texFragShd, attrib0='position')
        self._plotFBOs = {}

        self._keepDataAspectRatio = False

        self._crosshairCursor = None
        self._mousePosInPixels = None

        self._markers = OrderedDict()
        self._items = OrderedDict()
        self._plotContent = PlotDataContent()  # For images and curves
        self._glGarbageCollector = []

        self._plotFrame = GLPlotFrame2D(
            margins={'left': 100, 'right': 50, 'top': 50, 'bottom': 50})

        # Make postRedisplay asynchronous using Qt signal
        self._sigPostRedisplay.connect(
            super(BackendOpenGL, self).postRedisplay,
            qt.Qt.QueuedConnection)

        self.setAutoFillBackground(False)
        self.setMouseTracking(True)

    # QWidget

    _MOUSE_BTNS = {1: 'left', 2: 'right', 4: 'middle'}

    def contextMenuEvent(self, event):
        """Override QWidget.contextMenuEvent to implement the context menu"""
        # Makes sure it is overridden (issue with PySide)
        BackendBase.BackendBase.contextMenuEvent(self, event)

    def sizeHint(self):
        return qt.QSize(8 * 80, 6 * 80)  # Mimic MatplotlibBackend

    def mousePressEvent(self, event):
        xPixel = event.x() * self.getDevicePixelRatio()
        yPixel = event.y() * self.getDevicePixelRatio()
        btn = self._MOUSE_BTNS[event.button()]
        self._plot.onMousePress(xPixel, yPixel, btn)
        event.accept()

    def mouseMoveEvent(self, event):
        xPixel = event.x() * self.getDevicePixelRatio()
        yPixel = event.y() * self.getDevicePixelRatio()

        # Handle crosshair
        inXPixel, inYPixel = self._mouseInPlotArea(xPixel, yPixel)
        isCursorInPlot = inXPixel == xPixel and inYPixel == yPixel

        previousMousePosInPixels = self._mousePosInPixels
        self._mousePosInPixels = (xPixel, yPixel) if isCursorInPlot else None
        if (self._crosshairCursor is not None and
                previousMousePosInPixels != self._mousePosInPixels):
            # Avoid replot when cursor remains outside plot area
            self._plot._setDirtyPlot(overlayOnly=True)

        self._plot.onMouseMove(xPixel, yPixel)
        event.accept()

    def mouseReleaseEvent(self, event):
        xPixel = event.x() * self.getDevicePixelRatio()
        yPixel = event.y() * self.getDevicePixelRatio()

        btn = self._MOUSE_BTNS[event.button()]
        self._plot.onMouseRelease(xPixel, yPixel, btn)
        event.accept()

    def wheelEvent(self, event):
        xPixel = event.x() * self.getDevicePixelRatio()
        yPixel = event.y() * self.getDevicePixelRatio()

        if hasattr(event, 'angleDelta'):  # Qt 5
            delta = event.angleDelta().y()
        else:  # Qt 4 support
            delta = event.delta()
        angleInDegrees = delta / 8.
        self._plot.onMouseWheel(xPixel, yPixel, angleInDegrees)
        event.accept()

    def leaveEvent(self, _):
        self._plot.onMouseLeaveWidget()

    # OpenGLWidget API

    def initializeGL(self):
        gl.testGL()

        gl.glClearColor(1., 1., 1., 1.)
        gl.glClearStencil(0)

        gl.glEnable(gl.GL_BLEND)
        # gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
        gl.glBlendFuncSeparate(gl.GL_SRC_ALPHA,
                               gl.GL_ONE_MINUS_SRC_ALPHA,
                               gl.GL_ONE,
                               gl.GL_ONE)

        # For lines
        gl.glHint(gl.GL_LINE_SMOOTH_HINT, gl.GL_NICEST)

        # For points
        gl.glEnable(gl.GL_VERTEX_PROGRAM_POINT_SIZE)  # OpenGL 2
        gl.glEnable(gl.GL_POINT_SPRITE)  # OpenGL 2
        # gl.glEnable(gl.GL_PROGRAM_POINT_SIZE)

    def _paintDirectGL(self):
        self._renderPlotAreaGL()
        self._plotFrame.render()
        self._renderMarkersGL()
        self._renderOverlayGL()

    def _paintFBOGL(self):
        context = glu.getGLContext()
        plotFBOTex = self._plotFBOs.get(context)
        if (self._plot._getDirtyPlot() or self._plotFrame.isDirty or
                plotFBOTex is None):
            self._plotVertices = numpy.array(((-1., -1., 0., 0.),
                                             (1., -1., 1., 0.),
                                             (-1., 1., 0., 1.),
                                             (1., 1., 1., 1.)),
                                             dtype=numpy.float32)
            if plotFBOTex is None or \
               plotFBOTex.shape[1] != self._plotFrame.size[0] or \
               plotFBOTex.shape[0] != self._plotFrame.size[1]:
                if plotFBOTex is not None:
                    plotFBOTex.discard()
                plotFBOTex = glu.FramebufferTexture(
                    gl.GL_RGBA,
                    shape=(self._plotFrame.size[1],
                           self._plotFrame.size[0]),
                    minFilter=gl.GL_NEAREST,
                    magFilter=gl.GL_NEAREST,
                    wrap=(gl.GL_CLAMP_TO_EDGE,
                          gl.GL_CLAMP_TO_EDGE))
                self._plotFBOs[context] = plotFBOTex

            with plotFBOTex:
                gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_STENCIL_BUFFER_BIT)
                self._renderPlotAreaGL()
                self._plotFrame.render()

        # Render plot in screen coords
        gl.glViewport(0, 0, self._plotFrame.size[0], self._plotFrame.size[1])

        self._progTex.use()
        texUnit = 0

        gl.glUniform1i(self._progTex.uniforms['tex'], texUnit)
        gl.glUniformMatrix4fv(self._progTex.uniforms['matrix'], 1, gl.GL_TRUE,
                              mat4Identity().astype(numpy.float32))

        stride = self._plotVertices.shape[-1] * self._plotVertices.itemsize
        gl.glEnableVertexAttribArray(self._progTex.attributes['position'])
        gl.glVertexAttribPointer(self._progTex.attributes['position'],
                                 2,
                                 gl.GL_FLOAT,
                                 gl.GL_FALSE,
                                 stride, self._plotVertices)

        texCoordsPtr = c_void_p(self._plotVertices.ctypes.data +
                                2 * self._plotVertices.itemsize)  # Better way?
        gl.glEnableVertexAttribArray(self._progTex.attributes['texCoords'])
        gl.glVertexAttribPointer(self._progTex.attributes['texCoords'],
                                 2,
                                 gl.GL_FLOAT,
                                 gl.GL_FALSE,
                                 stride, texCoordsPtr)

        with plotFBOTex.texture:
            gl.glDrawArrays(gl.GL_TRIANGLE_STRIP, 0, len(self._plotVertices))

        self._renderMarkersGL()
        self._renderOverlayGL()

    def paintGL(self):
        global _current_context
        _current_context = self.context()

        glu.setGLContextGetter(_getContext)

        # Release OpenGL resources
        for item in self._glGarbageCollector:
            item.discard()
        self._glGarbageCollector = []

        gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_STENCIL_BUFFER_BIT)

        # Check if window is large enough
        plotWidth, plotHeight = self.getPlotBoundsInPixels()[2:]
        if plotWidth <= 2 or plotHeight <= 2:
            return

        # self._paintDirectGL()
        self._paintFBOGL()

        glu.setGLContextGetter()
        _current_context = None

    def _nonOrthoAxesLineMarkerPrimitives(self, marker, pixelOffset):
        """Generates the vertices and label for a line marker.

        :param dict marker: Description of a line marker
        :param int pixelOffset: Offset of text from borders in pixels
        :return: Line vertices and Text label or None
        :rtype: 2-tuple (2x2 numpy.array of float, Text2D)
        """
        label, vertices = None, None

        xCoord, yCoord = marker['x'], marker['y']
        assert xCoord is None or yCoord is None  # Specific to line markers

        # Get plot corners in data coords
        plotLeft, plotTop, plotWidth, plotHeight = self.getPlotBoundsInPixels()

        corners = [(plotLeft, plotTop),
                   (plotLeft, plotTop + plotHeight),
                   (plotLeft + plotWidth, plotTop + plotHeight),
                   (plotLeft + plotWidth, plotTop)]
        corners = numpy.array([self.pixelToData(x, y, axis='left', check=False)
                               for (x, y) in corners])

        borders = {
            'right': (corners[3], corners[2]),
            'top': (corners[0], corners[3]),
            'bottom': (corners[2], corners[1]),
            'left': (corners[1], corners[0])
        }

        textLayouts = {  # align, valign, offsets
            'right': (RIGHT, BOTTOM, (-1., -1.)),
            'top': (LEFT, TOP, (1., 1.)),
            'bottom': (LEFT, BOTTOM, (1., -1.)),
            'left': (LEFT, BOTTOM, (1., -1.))
        }

        if xCoord is None:  # Horizontal line in data space
            if marker['text'] is not None:
                # Find intersection of hline with borders in data
                # Order is important as it stops at first intersection
                for border_name in ('right', 'top', 'bottom', 'left'):
                    (x0, y0), (x1, y1) = borders[border_name]

                    if min(y0, y1) <= yCoord < max(y0, y1):
                        xIntersect = (yCoord - y0) * (x1 - x0) / (y1 - y0) + x0

                        # Add text label
                        pixelPos = self.dataToPixel(
                            xIntersect, yCoord, axis='left', check=False)

                        align, valign, offsets = textLayouts[border_name]

                        x = pixelPos[0] + offsets[0] * pixelOffset
                        y = pixelPos[1] + offsets[1] * pixelOffset
                        label = Text2D(marker['text'], x, y,
                                       color=marker['color'],
                                       bgColor=(1., 1., 1., 0.5),
                                       align=align, valign=valign)
                        break  # Stop at first intersection

            xMin, xMax = corners[:, 0].min(), corners[:, 0].max()
            vertices = numpy.array(
                ((xMin, yCoord), (xMax, yCoord)), dtype=numpy.float32)

        else:  # yCoord is None: vertical line in data space
            if marker['text'] is not None:
                # Find intersection of hline with borders in data
                # Order is important as it stops at first intersection
                for border_name in ('top', 'bottom', 'right', 'left'):
                    (x0, y0), (x1, y1) = borders[border_name]
                    if min(x0, x1) <= xCoord < max(x0, x1):
                        yIntersect = (xCoord - x0) * (y1 - y0) / (x1 - x0) + y0

                        # Add text label
                        pixelPos = self.dataToPixel(
                            xCoord, yIntersect, axis='left', check=False)

                        align, valign, offsets = textLayouts[border_name]

                        x = pixelPos[0] + offsets[0] * pixelOffset
                        y = pixelPos[1] + offsets[1] * pixelOffset
                        label = Text2D(marker['text'], x, y,
                                       color=marker['color'],
                                       bgColor=(1., 1., 1., 0.5),
                                       align=align, valign=valign)
                        break  # Stop at first intersection

            yMin, yMax = corners[:, 1].min(), corners[:, 1].max()
            vertices = numpy.array(
                ((xCoord, yMin), (xCoord, yMax)), dtype=numpy.float32)

        return vertices, label

    def _renderMarkersGL(self):
        if len(self._markers) == 0:
            return

        plotWidth, plotHeight = self.getPlotBoundsInPixels()[2:]

        # Render in plot area
        gl.glScissor(self._plotFrame.margins.left,
                     self._plotFrame.margins.bottom,
                     plotWidth, plotHeight)
        gl.glEnable(gl.GL_SCISSOR_TEST)

        gl.glViewport(0, 0, self._plotFrame.size[0], self._plotFrame.size[1])

        # Prepare vertical and horizontal markers rendering
        self._progBase.use()
        gl.glUniformMatrix4fv(
            self._progBase.uniforms['matrix'], 1, gl.GL_TRUE,
            self.matScreenProj.astype(numpy.float32))
        gl.glUniform2i(self._progBase.uniforms['isLog'], False, False)
        gl.glUniform1i(self._progBase.uniforms['hatchStep'], 0)
        gl.glUniform1f(self._progBase.uniforms['tickLen'], 0.)
        posAttrib = self._progBase.attributes['position']

        labels = []
        pixelOffset = 3

        for marker in self._markers.values():
            xCoord, yCoord = marker['x'], marker['y']

            if ((self._plotFrame.xAxis.isLog and
                    xCoord is not None and
                    xCoord <= 0) or
                    (self._plotFrame.yAxis.isLog and
                    yCoord is not None and
                    yCoord <= 0)):
                # Do not render markers with negative coords on log axis
                continue

            if xCoord is None or yCoord is None:
                if not self.isDefaultBaseVectors():  # Non-orthogonal axes
                    vertices, label = self._nonOrthoAxesLineMarkerPrimitives(
                        marker, pixelOffset)
                    if label is not None:
                        labels.append(label)

                else:  # Orthogonal axes
                    pixelPos = self.dataToPixel(
                        xCoord, yCoord, axis='left', check=False)

                    if xCoord is None:  # Horizontal line in data space
                        if marker['text'] is not None:
                            x = self._plotFrame.size[0] - \
                                self._plotFrame.margins.right - pixelOffset
                            y = pixelPos[1] - pixelOffset
                            label = Text2D(marker['text'], x, y,
                                           color=marker['color'],
                                           bgColor=(1., 1., 1., 0.5),
                                           align=RIGHT, valign=BOTTOM)
                            labels.append(label)

                        width = self._plotFrame.size[0]
                        vertices = numpy.array(((0, pixelPos[1]),
                                                (width, pixelPos[1])),
                                               dtype=numpy.float32)

                    else:  # yCoord is None: vertical line in data space
                        if marker['text'] is not None:
                            x = pixelPos[0] + pixelOffset
                            y = self._plotFrame.margins.top + pixelOffset
                            label = Text2D(marker['text'], x, y,
                                           color=marker['color'],
                                           bgColor=(1., 1., 1., 0.5),
                                           align=LEFT, valign=TOP)
                            labels.append(label)

                        height = self._plotFrame.size[1]
                        vertices = numpy.array(((pixelPos[0], 0),
                                                (pixelPos[0], height)),
                                               dtype=numpy.float32)

                self._progBase.use()
                gl.glUniform4f(self._progBase.uniforms['color'],
                               *marker['color'])

                gl.glEnableVertexAttribArray(posAttrib)
                gl.glVertexAttribPointer(posAttrib,
                                         2,
                                         gl.GL_FLOAT,
                                         gl.GL_FALSE,
                                         0, vertices)
                gl.glLineWidth(1)
                gl.glDrawArrays(gl.GL_LINES, 0, len(vertices))

            else:
                pixelPos = self.dataToPixel(
                    xCoord, yCoord, axis='left', check=True)
                if pixelPos is None:
                    # Do not render markers outside visible plot area
                    continue

                if marker['text'] is not None:
                    x = pixelPos[0] + pixelOffset
                    y = pixelPos[1] + pixelOffset
                    label = Text2D(marker['text'], x, y,
                                   color=marker['color'],
                                   bgColor=(1., 1., 1., 0.5),
                                   align=LEFT, valign=TOP)
                    labels.append(label)

                # For now simple implementation: using a curve for each marker
                # Should pack all markers to a single set of points
                markerCurve = GLPlotCurve2D(
                    numpy.array((pixelPos[0],), dtype=numpy.float64),
                    numpy.array((pixelPos[1],), dtype=numpy.float64),
                    marker=marker['symbol'],
                    markerColor=marker['color'],
                    markerSize=11)
                markerCurve.render(self.matScreenProj, False, False)
                markerCurve.discard()

        gl.glViewport(0, 0, self._plotFrame.size[0], self._plotFrame.size[1])

        # Render marker labels
        for label in labels:
            label.render(self.matScreenProj)

        gl.glDisable(gl.GL_SCISSOR_TEST)

    def _renderOverlayGL(self):
        # Render crosshair cursor
        if self._crosshairCursor is not None:
            plotWidth, plotHeight = self.getPlotBoundsInPixels()[2:]

            # Scissor to plot area
            gl.glScissor(self._plotFrame.margins.left,
                         self._plotFrame.margins.bottom,
                         plotWidth, plotHeight)
            gl.glEnable(gl.GL_SCISSOR_TEST)

            self._progBase.use()
            gl.glUniform2i(self._progBase.uniforms['isLog'], False, False)
            gl.glUniform1f(self._progBase.uniforms['tickLen'], 0.)
            posAttrib = self._progBase.attributes['position']
            matrixUnif = self._progBase.uniforms['matrix']
            colorUnif = self._progBase.uniforms['color']
            hatchStepUnif = self._progBase.uniforms['hatchStep']

            # Render crosshair cursor in screen frame but with scissor
            if (self._crosshairCursor is not None and
                    self._mousePosInPixels is not None):
                gl.glViewport(
                    0, 0, self._plotFrame.size[0], self._plotFrame.size[1])

                gl.glUniformMatrix4fv(matrixUnif, 1, gl.GL_TRUE,
                                      self.matScreenProj.astype(numpy.float32))

                color, lineWidth = self._crosshairCursor
                gl.glUniform4f(colorUnif, *color)
                gl.glUniform1i(hatchStepUnif, 0)

                xPixel, yPixel = self._mousePosInPixels
                xPixel, yPixel = xPixel + 0.5, yPixel + 0.5
                vertices = numpy.array(((0., yPixel),
                                        (self._plotFrame.size[0], yPixel),
                                        (xPixel, 0.),
                                        (xPixel, self._plotFrame.size[1])),
                                       dtype=numpy.float32)

                gl.glEnableVertexAttribArray(posAttrib)
                gl.glVertexAttribPointer(posAttrib,
                                         2,
                                         gl.GL_FLOAT,
                                         gl.GL_FALSE,
                                         0, vertices)
                gl.glLineWidth(lineWidth)
                gl.glDrawArrays(gl.GL_LINES, 0, len(vertices))

            gl.glDisable(gl.GL_SCISSOR_TEST)

    def _renderPlotAreaGL(self):
        plotWidth, plotHeight = self.getPlotBoundsInPixels()[2:]

        self._plotFrame.renderGrid()

        gl.glScissor(self._plotFrame.margins.left,
                     self._plotFrame.margins.bottom,
                     plotWidth, plotHeight)
        gl.glEnable(gl.GL_SCISSOR_TEST)

        # Matrix
        trBounds = self._plotFrame.transformedDataRanges
        if trBounds.x[0] == trBounds.x[1] or \
           trBounds.y[0] == trBounds.y[1]:
            return

        isXLog = self._plotFrame.xAxis.isLog
        isYLog = self._plotFrame.yAxis.isLog

        gl.glViewport(self._plotFrame.margins.left,
                      self._plotFrame.margins.bottom,
                      plotWidth, plotHeight)

        # Render images and curves
        # sorted is stable: original order is preserved when key is the same
        for item in self._plotContent.zOrderedPrimitives():
            if item.info.get('yAxis') == 'right':
                item.render(self._plotFrame.transformedDataY2ProjMat,
                            isXLog, isYLog)
            else:
                item.render(self._plotFrame.transformedDataProjMat,
                            isXLog, isYLog)

        # Render Items
        gl.glViewport(0, 0, self._plotFrame.size[0], self._plotFrame.size[1])

        self._progBase.use()
        gl.glUniformMatrix4fv(self._progBase.uniforms['matrix'], 1, gl.GL_TRUE,
                              self.matScreenProj.astype(numpy.float32))
        gl.glUniform2i(self._progBase.uniforms['isLog'], False, False)
        gl.glUniform1f(self._progBase.uniforms['tickLen'], 0.)

        for item in self._items.values():
            if ((isXLog and numpy.min(item['x']) < FLOAT32_MINPOS) or
                    (isYLog and numpy.min(item['y']) < FLOAT32_MINPOS)):
                # Ignore items <= 0. on log axes
                continue

            closed = item['shape'] != 'polylines'
            points = [self.dataToPixel(x, y, axis='left', check=False)
                      for (x, y) in zip(item['x'], item['y'])]
            shape2D = Shape2D(points,
                              fill=item['fill'],
                              fillColor=item['color'],
                              stroke=True,
                              strokeColor=item['color'],
                              strokeClosed=closed)

            posAttrib = self._progBase.attributes['position']
            colorUnif = self._progBase.uniforms['color']
            hatchStepUnif = self._progBase.uniforms['hatchStep']
            shape2D.render(posAttrib, colorUnif, hatchStepUnif)

        gl.glDisable(gl.GL_SCISSOR_TEST)

    def resizeGL(self, width, height):
        if width == 0 or height == 0:  # Do not resize
            return

        self._plotFrame.size = (
            int(self.getDevicePixelRatio() * width),
            int(self.getDevicePixelRatio() * height))

        self.matScreenProj = mat4Ortho(0, self._plotFrame.size[0],
                                       self._plotFrame.size[1], 0,
                                       1, -1)

        # Store current ranges
        previousXRange = self.getGraphXLimits()
        previousYRange = self.getGraphYLimits(axis='left')
        previousYRightRange = self.getGraphYLimits(axis='right')

        (xMin, xMax), (yMin, yMax), (y2Min, y2Max) = \
            self._plotFrame.dataRanges
        self.setLimits(xMin, xMax, yMin, yMax, y2Min, y2Max)

        # If plot range has changed, then emit signal
        if previousXRange != self.getGraphXLimits():
            self._plot.getXAxis()._emitLimitsChanged()
        if previousYRange != self.getGraphYLimits(axis='left'):
            self._plot.getYAxis(axis='left')._emitLimitsChanged()
        if previousYRightRange != self.getGraphYLimits(axis='right'):
            self._plot.getYAxis(axis='right')._emitLimitsChanged()

    # Add methods

    @staticmethod
    def _castArrayTo(v):
        """Returns best floating type to cast the array to.

        :param numpy.ndarray v: Array to cast
        :rtype: numpy.dtype
        :raise ValueError: If dtype is not supported
        """
        if numpy.issubdtype(v.dtype, numpy.floating):
            return numpy.float32 if v.itemsize <= 4 else numpy.float64
        elif numpy.issubdtype(v.dtype, numpy.integer):
            return numpy.float32 if v.itemsize <= 2 else numpy.float64
        else:
            raise ValueError('Unsupported data type')

    def addCurve(self, x, y, legend,
                 color, symbol, linewidth, linestyle,
                 yaxis,
                 xerror, yerror, z, selectable,
                 fill, alpha, symbolsize):
        for parameter in (x, y, legend, color, symbol, linewidth, linestyle,
                          yaxis, z, selectable, fill, symbolsize):
            assert parameter is not None
        assert yaxis in ('left', 'right')

        # Convert input data
        x = numpy.array(x, copy=False)
        y = numpy.array(y, copy=False)

        # Check if float32 is enough
        if (self._castArrayTo(x) is numpy.float32 and
                self._castArrayTo(y) is numpy.float32):
            dtype = numpy.float32
        else:
            dtype = numpy.float64

        x = numpy.array(x, dtype=dtype, copy=False, order='C')
        y = numpy.array(y, dtype=dtype, copy=False, order='C')

        # Convert errors to float32
        if xerror is not None:
            xerror = numpy.array(
                xerror, dtype=numpy.float32, copy=False, order='C')
        if yerror is not None:
            yerror = numpy.array(
                yerror, dtype=numpy.float32, copy=False, order='C')

        # Handle axes log scale: convert data

        if self._plotFrame.xAxis.isLog:
            logX = numpy.log10(x)

            if xerror is not None:
                # Transform xerror so that
                # log10(x) +/- xerror' = log10(x +/- xerror)
                if hasattr(xerror, 'shape') and len(xerror.shape) == 2:
                    xErrorMinus, xErrorPlus = xerror[0], xerror[1]
                else:
                    xErrorMinus, xErrorPlus = xerror, xerror
                xErrorMinus = logX - numpy.log10(x - xErrorMinus)
                xErrorPlus = numpy.log10(x + xErrorPlus) - logX
                xerror = numpy.array((xErrorMinus, xErrorPlus),
                                     dtype=numpy.float32)

            x = logX

        isYLog = (yaxis == 'left' and self._plotFrame.yAxis.isLog) or (
            yaxis == 'right' and self._plotFrame.y2Axis.isLog)

        if isYLog:
            logY = numpy.log10(y)

            if yerror is not None:
                # Transform yerror so that
                # log10(y) +/- yerror' = log10(y +/- yerror)
                if hasattr(yerror, 'shape') and len(yerror.shape) == 2:
                    yErrorMinus, yErrorPlus = yerror[0], yerror[1]
                else:
                    yErrorMinus, yErrorPlus = yerror, yerror
                yErrorMinus = logY - numpy.log10(y - yErrorMinus)
                yErrorPlus = numpy.log10(y + yErrorPlus) - logY
                yerror = numpy.array((yErrorMinus, yErrorPlus),
                                     dtype=numpy.float32)

            y = logY

        # TODO check if need more filtering of error (e.g., clip to positive)

        # TODO check and improve this
        if (len(color) == 4 and
                type(color[3]) in [type(1), numpy.uint8, numpy.int8]):
            color = numpy.array(color, dtype=numpy.float32) / 255.

        if isinstance(color, numpy.ndarray) and color.ndim == 2:
            colorArray = color
            color = None
        else:
            colorArray = None
            color = colors.rgba(color)

        if alpha < 1.:  # Apply image transparency
            if colorArray is not None and colorArray.shape[1] == 4:
                # multiply alpha channel
                colorArray[:, 3] = colorArray[:, 3] * alpha
            if color is not None:
                color = color[0], color[1], color[2], color[3] * alpha

        behaviors = set()
        if selectable:
            behaviors.add('selectable')

        curve = GLPlotCurve2D(x, y, colorArray,
                              xError=xerror,
                              yError=yerror,
                              lineStyle=linestyle,
                              lineColor=color,
                              lineWidth=linewidth,
                              marker=symbol,
                              markerColor=color,
                              markerSize=symbolsize,
                              fillColor=color if fill else None,
                              isYLog=isYLog)
        curve.info = {
            'legend': legend,
            'zOrder': z,
            'behaviors': behaviors,
            'yAxis': 'left' if yaxis is None else yaxis,
        }

        if yaxis == "right":
            self._plotFrame.isY2Axis = True

        self._plotContent.add(curve)

        return legend, 'curve'

    def addImage(self, data, legend,
                 origin, scale, z,
                 selectable, draggable,
                 colormap, alpha):
        for parameter in (data, legend, origin, scale, z,
                          selectable, draggable):
            assert parameter is not None

        behaviors = set()
        if selectable:
            behaviors.add('selectable')
        if draggable:
            behaviors.add('draggable')

        if data.ndim == 2:
            # Ensure array is contiguous and eventually convert its type
            if data.dtype in (numpy.float32, numpy.uint8, numpy.uint16):
                data = numpy.array(data, copy=False, order='C')
            else:
                _logger.info(
                    'addImage: Convert %s data to float32', str(data.dtype))
                data = numpy.array(data, dtype=numpy.float32, order='C')

            colormapIsLog = colormap.getNormalization() == 'log'
            cmapRange = colormap.getColormapRange(data=data)
            colormapLut = colormap.getNColors(nbColors=256)

            image = GLPlotColormap(data,
                                   origin,
                                   scale,
                                   colormapLut,
                                   colormapIsLog,
                                   cmapRange,
                                   alpha)
            image.info = {
                'legend': legend,
                'zOrder': z,
                'behaviors': behaviors
            }
            self._plotContent.add(image)

        elif len(data.shape) == 3:
            # For RGB, RGBA data
            assert data.shape[2] in (3, 4)

            if numpy.issubdtype(data.dtype, numpy.floating):
                data = numpy.array(data, dtype=numpy.float32, copy=False)
            elif numpy.issubdtype(data.dtype, numpy.integer):
                data = numpy.array(data, dtype=numpy.uint8, copy=False)
            else:
                raise ValueError('Unsupported data type')

            image = GLPlotRGBAImage(data, origin, scale, alpha)

            image.info = {
                'legend': legend,
                'zOrder': z,
                'behaviors': behaviors
            }

            if self._plotFrame.xAxis.isLog and image.xMin <= 0.:
                raise RuntimeError(
                    'Cannot add image with X <= 0 with X axis log scale')
            if self._plotFrame.yAxis.isLog and image.yMin <= 0.:
                raise RuntimeError(
                    'Cannot add image with Y <= 0 with Y axis log scale')

            self._plotContent.add(image)

        else:
            raise RuntimeError("Unsupported data shape {0}".format(data.shape))

        return legend, 'image'

    def addItem(self, x, y, legend, shape, color, fill, overlay, z):
        # TODO handle overlay
        if shape not in ('polygon', 'rectangle', 'line',
                         'vline', 'hline', 'polylines'):
            raise NotImplementedError("Unsupported shape {0}".format(shape))

        x = numpy.array(x, copy=False)
        y = numpy.array(y, copy=False)

        if shape == 'rectangle':
            xMin, xMax = x
            x = numpy.array((xMin, xMin, xMax, xMax))
            yMin, yMax = y
            y = numpy.array((yMin, yMax, yMax, yMin))

        # TODO is this needed?
        if self._plotFrame.xAxis.isLog and x.min() <= 0.:
            raise RuntimeError(
                'Cannot add item with X <= 0 with X axis log scale')
        if self._plotFrame.yAxis.isLog and y.min() <= 0.:
            raise RuntimeError(
                'Cannot add item with Y <= 0 with Y axis log scale')

        # Ignore fill for polylines to mimic matplotlib
        fill = fill if shape != 'polylines' else False

        self._items[legend] = {
            'shape': shape,
            'color': colors.rgba(color),
            'fill': 'hatch' if fill else None,
            'x': x,
            'y': y
        }

        return legend, 'item'

    def addMarker(self, x, y, legend, text, color,
                  selectable, draggable,
                  symbol, constraint):

        if symbol is None:
            symbol = '+'

        behaviors = set()
        if selectable:
            behaviors.add('selectable')
        if draggable:
            behaviors.add('draggable')

        # Apply constraint to provided position
        isConstraint = (draggable and constraint is not None and
                        x is not None and y is not None)
        if isConstraint:
            x, y = constraint(x, y)

        self._markers[legend] = {
            'x': x,
            'y': y,
            'legend': legend,
            'text': text,
            'color': colors.rgba(color),
            'behaviors': behaviors,
            'constraint': constraint if isConstraint else None,
            'symbol': symbol,
        }

        return legend, 'marker'

    # Remove methods

    def remove(self, item):
        legend, kind = item

        if kind == 'curve':
            curve = self._plotContent.pop('curve', legend)
            if curve is not None:
                # Check if some curves remains on the right Y axis
                y2AxisItems = (item for item in self._plotContent.primitives()
                               if item.info.get('yAxis', 'left') == 'right')
                self._plotFrame.isY2Axis = next(y2AxisItems, None) is not None

                self._glGarbageCollector.append(curve)

        elif kind == 'image':
            image = self._plotContent.pop('image', legend)
            if image is not None:
                self._glGarbageCollector.append(image)

        elif kind == 'marker':
            self._markers.pop(legend, False)

        elif kind == 'item':
            self._items.pop(legend, False)

        else:
            _logger.error('Unsupported kind: %s', str(kind))

    # Interaction methods

    _QT_CURSORS = {
        None: qt.Qt.ArrowCursor,
        BackendBase.CURSOR_DEFAULT: qt.Qt.ArrowCursor,
        BackendBase.CURSOR_POINTING: qt.Qt.PointingHandCursor,
        BackendBase.CURSOR_SIZE_HOR: qt.Qt.SizeHorCursor,
        BackendBase.CURSOR_SIZE_VER: qt.Qt.SizeVerCursor,
        BackendBase.CURSOR_SIZE_ALL: qt.Qt.SizeAllCursor,
    }

    def setGraphCursorShape(self, cursor):
        cursor = self._QT_CURSORS[cursor]

        super(BackendOpenGL, self).setCursor(qt.QCursor(cursor))

    def setGraphCursor(self, flag, color, linewidth, linestyle):
        if linestyle is not '-':
            _logger.warning(
                "BackendOpenGL.setGraphCursor linestyle parameter ignored")

        if flag:
            color = colors.rgba(color)
            crosshairCursor = color, linewidth
        else:
            crosshairCursor = None

        if crosshairCursor != self._crosshairCursor:
            self._crosshairCursor = crosshairCursor

    _PICK_OFFSET = 3  # Offset in pixel used for picking

    def _mouseInPlotArea(self, x, y):
        xPlot = numpy.clip(
            x, self._plotFrame.margins.left,
            self._plotFrame.size[0] - self._plotFrame.margins.right - 1)
        yPlot = numpy.clip(
            y, self._plotFrame.margins.top,
            self._plotFrame.size[1] - self._plotFrame.margins.bottom - 1)
        return xPlot, yPlot

    def pickItems(self, x, y, kinds):
        picked = []

        dataPos = self.pixelToData(x, y, axis='left', check=True)
        if dataPos is not None:
            # Pick markers
            if 'marker' in kinds:
                for marker in reversed(list(self._markers.values())):
                    pixelPos = self.dataToPixel(
                        marker['x'], marker['y'], axis='left', check=False)
                    if pixelPos is None:  # negative coord on a log axis
                        continue

                    if marker['x'] is None:  # Horizontal line
                        pt1 = self.pixelToData(
                            x, y - self._PICK_OFFSET, axis='left', check=False)
                        pt2 = self.pixelToData(
                            x, y + self._PICK_OFFSET, axis='left', check=False)
                        isPicked = (min(pt1[1], pt2[1]) <= marker['y'] <=
                                    max(pt1[1], pt2[1]))

                    elif marker['y'] is None:  # Vertical line
                        pt1 = self.pixelToData(
                            x - self._PICK_OFFSET, y, axis='left', check=False)
                        pt2 = self.pixelToData(
                            x + self._PICK_OFFSET, y, axis='left', check=False)
                        isPicked = (min(pt1[0], pt2[0]) <= marker['x'] <=
                                    max(pt1[0], pt2[0]))

                    else:
                        isPicked = (
                            numpy.fabs(x - pixelPos[0]) <= self._PICK_OFFSET and
                            numpy.fabs(y - pixelPos[1]) <= self._PICK_OFFSET)

                    if isPicked:
                        picked.append(dict(kind='marker',
                                           legend=marker['legend']))

            # Pick image and curves
            if 'image' in kinds or 'curve' in kinds:
                for item in self._plotContent.zOrderedPrimitives(reverse=True):
                    if ('image' in kinds and
                            isinstance(item, (GLPlotColormap, GLPlotRGBAImage))):
                        pickedPos = item.pick(*dataPos)
                        if pickedPos is not None:
                            picked.append(dict(kind='image',
                                               legend=item.info['legend']))

                    elif 'curve' in kinds and isinstance(item, GLPlotCurve2D):
                        offset = self._PICK_OFFSET
                        if item.marker is not None:
                            offset = max(item.markerSize / 2., offset)
                        if item.lineStyle is not None:
                            offset = max(item.lineWidth / 2., offset)

                        yAxis = item.info['yAxis']

                        inAreaPos = self._mouseInPlotArea(x - offset, y - offset)
                        dataPos = self.pixelToData(inAreaPos[0], inAreaPos[1],
                                                   axis=yAxis, check=True)
                        if dataPos is None:
                            continue
                        xPick0, yPick0 = dataPos

                        inAreaPos = self._mouseInPlotArea(x + offset, y + offset)
                        dataPos = self.pixelToData(inAreaPos[0], inAreaPos[1],
                                                   axis=yAxis, check=True)
                        if dataPos is None:
                            continue
                        xPick1, yPick1 = dataPos

                        if xPick0 < xPick1:
                            xPickMin, xPickMax = xPick0, xPick1
                        else:
                            xPickMin, xPickMax = xPick1, xPick0

                        if yPick0 < yPick1:
                            yPickMin, yPickMax = yPick0, yPick1
                        else:
                            yPickMin, yPickMax = yPick1, yPick0

                        # Apply log scale if axis is log
                        if self._plotFrame.xAxis.isLog:
                            xPickMin = numpy.log10(xPickMin)
                            xPickMax = numpy.log10(xPickMax)

                        if (yAxis == 'left' and self._plotFrame.yAxis.isLog) or (
                                yAxis == 'right' and self._plotFrame.y2Axis.isLog):
                            yPickMin = numpy.log10(yPickMin)
                            yPickMax = numpy.log10(yPickMax)

                        pickedIndices = item.pick(xPickMin, yPickMin,
                                                  xPickMax, yPickMax)
                        if pickedIndices:
                            picked.append(dict(kind='curve',
                                               legend=item.info['legend'],
                                               indices=pickedIndices))

        return picked

    # Update curve

    def setCurveColor(self, curve, color):
        pass  # TODO

    # Misc.

    def getWidgetHandle(self):
        return self

    def postRedisplay(self):
        self._sigPostRedisplay.emit()

    def replot(self):
        self.update()  # async redraw
        # self.repaint()  # immediate redraw

    def saveGraph(self, fileName, fileFormat, dpi):
        if dpi is not None:
            _logger.warning("saveGraph ignores dpi parameter")

        if fileFormat not in ['png', 'ppm', 'svg', 'tiff']:
            raise NotImplementedError('Unsupported format: %s' % fileFormat)

        if not self.isValid():
            _logger.error('OpenGL 2.1 not available, cannot save OpenGL image')
            width, height = self._plotFrame.size
            data = numpy.zeros((height, width, 3), dtype=numpy.uint8)
        else:
            self.makeCurrent()

            data = numpy.empty(
                (self._plotFrame.size[1], self._plotFrame.size[0], 3),
                dtype=numpy.uint8, order='C')

            context = self.context()
            framebufferTexture = self._plotFBOs.get(context)
            if framebufferTexture is None:
                # Fallback, supports direct rendering mode: _paintDirectGL
                # might have issues as it can read on-screen framebuffer
                fboName = self.defaultFramebufferObject()
                width, height = self._plotFrame.size
            else:
                fboName = framebufferTexture.name
                height, width = framebufferTexture.shape

            previousFramebuffer = gl.glGetInteger(gl.GL_FRAMEBUFFER_BINDING)
            gl.glBindFramebuffer(gl.GL_FRAMEBUFFER, fboName)
            gl.glPixelStorei(gl.GL_PACK_ALIGNMENT, 1)
            gl.glReadPixels(0, 0, width, height,
                            gl.GL_RGB, gl.GL_UNSIGNED_BYTE, data)
            gl.glBindFramebuffer(gl.GL_FRAMEBUFFER, previousFramebuffer)

            # glReadPixels gives bottom to top,
            # while images are stored as top to bottom
            data = numpy.flipud(data)

        # fileName is either a file-like object or a str
        saveImageToFile(data, fileName, fileFormat)

    # Graph labels

    def setGraphTitle(self, title):
        self._plotFrame.title = title

    def setGraphXLabel(self, label):
        self._plotFrame.xAxis.title = label

    def setGraphYLabel(self, label, axis):
        if axis == 'left':
            self._plotFrame.yAxis.title = label
        else:  # right axis
            if label:
                _logger.warning('Right axis label not implemented')

    # Non orthogonal axes

    def setBaseVectors(self, x=(1., 0.), y=(0., 1.)):
        """Set base vectors.

        Useful for non-orthogonal axes.
        If an axis is in log scale, skew is applied to log transformed values.

        Base vector does not work well with log axes, to investi
        """
        if x != (1., 0.) and y != (0., 1.):
            if self._plotFrame.xAxis.isLog:
                _logger.warning("setBaseVectors disables X axis logarithmic.")
                self.setXAxisLogarithmic(False)
            if self._plotFrame.yAxis.isLog:
                _logger.warning("setBaseVectors disables Y axis logarithmic.")
                self.setYAxisLogarithmic(False)

            if self.isKeepDataAspectRatio():
                _logger.warning("setBaseVectors disables keepDataAspectRatio.")
                self.keepDataAspectRatio(False)

        self._plotFrame.baseVectors = x, y

    def getBaseVectors(self):
        return self._plotFrame.baseVectors

    def isDefaultBaseVectors(self):
        return self._plotFrame.baseVectors == \
            self._plotFrame.DEFAULT_BASE_VECTORS

    # Graph limits

    def _setDataRanges(self, xlim=None, ylim=None, y2lim=None):
        """Set the visible range of data in the plot frame.

        This clips the ranges to possible values (takes care of float32
        range + positive range for log).
        This also takes care of non-orthogonal axes.

        This should be moved to PlotFrame.
        """
        # Update axes range with a clipped range if too wide
        self._plotFrame.setDataRanges(xlim, ylim, y2lim)

        if not self.isDefaultBaseVectors():
            # Update axes range with axes bounds in data coords
            plotLeft, plotTop, plotWidth, plotHeight = \
                self.getPlotBoundsInPixels()

            self._plotFrame.xAxis.dataRange = sorted([
                self.pixelToData(x, y, axis='left', check=False)[0]
                for (x, y) in ((plotLeft, plotTop + plotHeight),
                               (plotLeft + plotWidth, plotTop + plotHeight))])

            self._plotFrame.yAxis.dataRange = sorted([
                self.pixelToData(x, y, axis='left', check=False)[1]
                for (x, y) in ((plotLeft, plotTop + plotHeight),
                               (plotLeft, plotTop))])

            self._plotFrame.y2Axis.dataRange = sorted([
                self.pixelToData(x, y, axis='right', check=False)[1]
                for (x, y) in ((plotLeft + plotWidth, plotTop + plotHeight),
                               (plotLeft + plotWidth, plotTop))])

    def _ensureAspectRatio(self, keepDim=None):
        """Update plot bounds in order to keep aspect ratio.

        Warning: keepDim on right Y axis is not implemented !

        :param str keepDim: The dimension to maintain: 'x', 'y' or None.
            If None (the default), the dimension with the largest range.
        """
        plotWidth, plotHeight = self.getPlotBoundsInPixels()[2:]
        if plotWidth <= 2 or plotHeight <= 2:
            return

        if keepDim is None:
            dataBounds = self._plotContent.getBounds(
                self._plotFrame.xAxis.isLog, self._plotFrame.yAxis.isLog)
            if dataBounds.yAxis.range_ != 0.:
                dataRatio = dataBounds.xAxis.range_
                dataRatio /= float(dataBounds.yAxis.range_)

                plotRatio = plotWidth / float(plotHeight)  # Test != 0 before

                keepDim = 'x' if dataRatio > plotRatio else 'y'
            else:  # Limit case
                keepDim = 'x'

        (xMin, xMax), (yMin, yMax), (y2Min, y2Max) = \
            self._plotFrame.dataRanges
        if keepDim == 'y':
            dataW = (yMax - yMin) * plotWidth / float(plotHeight)
            xCenter = 0.5 * (xMin + xMax)
            xMin = xCenter - 0.5 * dataW
            xMax = xCenter + 0.5 * dataW
        elif keepDim == 'x':
            dataH = (xMax - xMin) * plotHeight / float(plotWidth)
            yCenter = 0.5 * (yMin + yMax)
            yMin = yCenter - 0.5 * dataH
            yMax = yCenter + 0.5 * dataH
            y2Center = 0.5 * (y2Min + y2Max)
            y2Min = y2Center - 0.5 * dataH
            y2Max = y2Center + 0.5 * dataH
        else:
            raise RuntimeError('Unsupported dimension to keep: %s' % keepDim)

        # Update plot frame bounds
        self._setDataRanges(xlim=(xMin, xMax),
                            ylim=(yMin, yMax),
                            y2lim=(y2Min, y2Max))

    def _setPlotBounds(self, xRange=None, yRange=None, y2Range=None,
                       keepDim=None):
        # Update axes range with a clipped range if too wide
        self._setDataRanges(xlim=xRange,
                            ylim=yRange,
                            y2lim=y2Range)

        # Keep data aspect ratio
        if self.isKeepDataAspectRatio():
            self._ensureAspectRatio(keepDim)

    def setLimits(self, xmin, xmax, ymin, ymax, y2min=None, y2max=None):
        assert xmin < xmax
        assert ymin < ymax

        if y2min is None or y2max is None:
            y2Range = None
        else:
            assert y2min < y2max
            y2Range = y2min, y2max
        self._setPlotBounds((xmin, xmax), (ymin, ymax), y2Range)

    def getGraphXLimits(self):
        return self._plotFrame.dataRanges.x

    def setGraphXLimits(self, xmin, xmax):
        assert xmin < xmax
        self._setPlotBounds(xRange=(xmin, xmax), keepDim='x')

    def getGraphYLimits(self, axis):
        assert axis in ("left", "right")
        if axis == "left":
            return self._plotFrame.dataRanges.y
        else:
            return self._plotFrame.dataRanges.y2

    def setGraphYLimits(self, ymin, ymax, axis):
        assert ymin < ymax
        assert axis in ("left", "right")

        if axis == "left":
            self._setPlotBounds(yRange=(ymin, ymax), keepDim='y')
        else:
            self._setPlotBounds(y2Range=(ymin, ymax), keepDim='y')

    # Graph axes

    def getXAxisTimeZone(self):
        return self._plotFrame.xAxis.timeZone

    def setXAxisTimeZone(self, tz):
        self._plotFrame.xAxis.timeZone = tz

    def isXAxisTimeSeries(self):
        return self._plotFrame.xAxis.isTimeSeries

    def setXAxisTimeSeries(self, isTimeSeries):
        self._plotFrame.xAxis.isTimeSeries = isTimeSeries

    def setXAxisLogarithmic(self, flag):
        if flag != self._plotFrame.xAxis.isLog:
            if flag and self._keepDataAspectRatio:
                _logger.warning(
                    "KeepDataAspectRatio is ignored with log axes")

            if flag and not self.isDefaultBaseVectors():
                _logger.warning(
                    "setXAxisLogarithmic ignored because baseVectors are set")
                return

            self._plotFrame.xAxis.isLog = flag

    def setYAxisLogarithmic(self, flag):
        if (flag != self._plotFrame.yAxis.isLog or
                flag != self._plotFrame.y2Axis.isLog):
            if flag and self._keepDataAspectRatio:
                _logger.warning(
                    "KeepDataAspectRatio is ignored with log axes")

            if flag and not self.isDefaultBaseVectors():
                _logger.warning(
                    "setYAxisLogarithmic ignored because baseVectors are set")
                return

            self._plotFrame.yAxis.isLog = flag
            self._plotFrame.y2Axis.isLog = flag

    def setYAxisInverted(self, flag):
        if flag != self._plotFrame.isYAxisInverted:
            self._plotFrame.isYAxisInverted = flag

    def isYAxisInverted(self):
        return self._plotFrame.isYAxisInverted

    def isKeepDataAspectRatio(self):
        if self._plotFrame.xAxis.isLog or self._plotFrame.yAxis.isLog:
            return False
        else:
            return self._keepDataAspectRatio

    def setKeepDataAspectRatio(self, flag):
        if flag and (self._plotFrame.xAxis.isLog or
                     self._plotFrame.yAxis.isLog):
            _logger.warning("KeepDataAspectRatio is ignored with log axes")
        if flag and not self.isDefaultBaseVectors():
            _logger.warning(
                "keepDataAspectRatio ignored because baseVectors are set")

        self._keepDataAspectRatio = flag

    def setGraphGrid(self, which):
        assert which in (None, 'major', 'both')
        self._plotFrame.grid = which is not None  # TODO True grid support

    # Data <-> Pixel coordinates conversion

    def dataToPixel(self, x, y, axis, check=False):
        assert axis in ('left', 'right')

        if x is None or y is None:
            dataBounds = self._plotContent.getBounds(
                self._plotFrame.xAxis.isLog, self._plotFrame.yAxis.isLog)

            if x is None:
                x = dataBounds.xAxis.center

            if y is None:
                if axis == 'left':
                    y = dataBounds.yAxis.center
                else:
                    y = dataBounds.y2Axis.center

        result = self._plotFrame.dataToPixel(x, y, axis)

        if check and result is not None:
            xPixel, yPixel = result
            width, height = self._plotFrame.size
            if (xPixel < self._plotFrame.margins.left or
                    xPixel > (width - self._plotFrame.margins.right) or
                    yPixel < self._plotFrame.margins.top or
                    yPixel > height - self._plotFrame.margins.bottom):
                return None  # (x, y) is out of plot area

        return result

    def pixelToData(self, x, y, axis, check):
        assert axis in ("left", "right")

        if x is None:
            x = self._plotFrame.size[0] / 2.
        if y is None:
            y = self._plotFrame.size[1] / 2.

        if check and (x < self._plotFrame.margins.left or
                      x > (self._plotFrame.size[0] -
                           self._plotFrame.margins.right) or
                      y < self._plotFrame.margins.top or
                      y > (self._plotFrame.size[1] -
                           self._plotFrame.margins.bottom)):
            return None  # (x, y) is out of plot area

        return self._plotFrame.pixelToData(x, y, axis)

    def getPlotBoundsInPixels(self):
        return self._plotFrame.plotOrigin + self._plotFrame.plotSize

    def setAxesDisplayed(self, displayed):
        BackendBase.BackendBase.setAxesDisplayed(self, displayed)
        self._plotFrame.displayed = displayed



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""
This module contains :class:`SafeIconProvider`.
"""

__authors__ = ["V. Valls"]
__license__ = "MIT"
__date__ = "31/10/2017"

import sys
import logging
from silx.gui import qt


_logger = logging.getLogger(__name__)


class SafeFileIconProvider(qt.QFileIconProvider):
    """
    This class reimplement :class:`qt.QFileIconProvider` to avoid blocking
    access to the file system.

    It avoid to use `qt.QFileInfo.absoluteFilePath` or
    `qt.QFileInfo.canonicalPath` to reach drive icons which are known to
    freeze the file system using network drives.

    Computer root, and drive root paths are filtered. Other paths are not
    filtered while it is anyway needed to synchronoze a drive to accesss to it.
    """

    WIN32_DRIVE_UNKNOWN = 0
    """The drive type cannot be determined."""
    WIN32_DRIVE_NO_ROOT_DIR = 1
    """The root path is invalid; for example, there is no volume mounted at the
    specified path."""
    WIN32_DRIVE_REMOVABLE = 2
    """The drive has removable media; for example, a floppy drive, thumb drive,
    or flash card reader."""
    WIN32_DRIVE_FIXED = 3
    """The drive has fixed media; for example, a hard disk drive or flash
    drive."""
    WIN32_DRIVE_REMOTE = 4
    """The drive is a remote (network) drive."""
    WIN32_DRIVE_CDROM = 5
    """The drive is a CD-ROM drive."""
    WIN32_DRIVE_RAMDISK = 6
    """The drive is a RAM disk."""

    def __init__(self):
        qt.QFileIconProvider.__init__(self)
        self.__filterDirAndFiles = False
        if sys.platform == "win32":
            self._windowsTypes = {}
            item = "Drive", qt.QStyle.SP_DriveHDIcon
            self._windowsTypes[self.WIN32_DRIVE_UNKNOWN] = item
            item = "Invalid root", qt.QStyle.SP_DriveHDIcon
            self._windowsTypes[self.WIN32_DRIVE_NO_ROOT_DIR] = item
            item = "Removable", qt.QStyle.SP_DriveNetIcon
            self._windowsTypes[self.WIN32_DRIVE_REMOVABLE] = item
            item = "Drive", qt.QStyle.SP_DriveHDIcon
            self._windowsTypes[self.WIN32_DRIVE_FIXED] = item
            item = "Remote", qt.QStyle.SP_DriveNetIcon
            self._windowsTypes[self.WIN32_DRIVE_REMOTE] = item
            item = "CD-ROM", qt.QStyle.SP_DriveCDIcon
            self._windowsTypes[self.WIN32_DRIVE_CDROM] = item
            item = "RAM disk", qt.QStyle.SP_DriveHDIcon
            self._windowsTypes[self.WIN32_DRIVE_RAMDISK] = item

    def __windowsDriveTypeId(self, info):
        try:
            import ctypes
            path = info.filePath()
            dtype = ctypes.cdll.kernel32.GetDriveTypeW(path)
        except Exception:
            _logger.warning("Impossible to identify drive %s" % path)
            _logger.debug("Backtrace", exc_info=True)
            return self.WIN32_DRIVE_UNKNOWN
        return dtype

    def __windowsDriveIcon(self, info):
        dtype = self.__windowsDriveTypeId(info)
        default = self._windowsTypes[self.WIN32_DRIVE_UNKNOWN]
        driveInfo = self._windowsTypes.get(dtype, default)
        style = qt.QApplication.instance().style()
        icon = style.standardIcon(driveInfo[1])
        return icon

    def __windowsDriveType(self, info):
        dtype = self.__windowsDriveTypeId(info)
        default = self._windowsTypes[self.WIN32_DRIVE_UNKNOWN]
        driveInfo = self._windowsTypes.get(dtype, default)
        return driveInfo[0]

    def icon(self, info):
        style = qt.QApplication.instance().style()
        path = info.filePath()
        if path in ["", "/"]:
            # That's the computer root on Windows or Linux
            result = style.standardIcon(qt.QStyle.SP_ComputerIcon)
        elif sys.platform == "win32" and path[-2] == ":":
            # That's a drive on Windows
            result = self.__windowsDriveIcon(info)
        elif self.__filterDirAndFiles:
            if info.isDir():
                result = style.standardIcon(qt.QStyle.SP_DirIcon)
            else:
                result = style.standardIcon(qt.QStyle.SP_FileIcon)
        else:
            result = qt.QFileIconProvider.icon(self, info)
        return result

    def type(self, info):
        path = info.filePath()
        if path in ["", "/"]:
            # That's the computer root on Windows or Linux
            result = "Computer"
        elif sys.platform == "win32" and path[-2] == ":":
            # That's a drive on Windows
            result = self.__windowsDriveType(info)
        elif self.__filterDirAndFiles:
            if info.isDir():
                result = "Directory"
            else:
                result = info.suffix()
        else:
            result = qt.QFileIconProvider.type(self, info)
        return result



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Basic tests for silx.gui.plot.tools package"""

__authors__ = ["T. Vincent"]
__license__ = "MIT"
__date__ = "02/03/2018"


import functools
import unittest
import numpy

from silx.utils.testutils import TestLogging
from silx.gui.test.utils import qWaitForWindowExposedAndActivate
from silx.gui import qt
from silx.gui.plot import PlotWindow
from silx.gui.plot import tools
from silx.gui.plot.test.utils import PlotWidgetTestCase


# Makes sure a QApplication exists
_qapp = qt.QApplication.instance() or qt.QApplication([])


def _tearDownDocTest(docTest):
    """Tear down to use for test from docstring.

    Checks that plot widget is displayed
    """
    plot = docTest.globs['plot']
    qWaitForWindowExposedAndActivate(plot)
    plot.setAttribute(qt.Qt.WA_DeleteOnClose)
    plot.close()
    del plot

# Disable doctest because of
# "NameError: name 'numpy' is not defined"
#
# import doctest
# positionInfoTestSuite = doctest.DocTestSuite(
#     PlotTools, tearDown=_tearDownDocTest,
#     optionflags=doctest.ELLIPSIS)
# """Test suite of tests from PlotTools docstrings.
#
# Test PositionInfo and ProfileToolBar docstrings.
# """


class TestPositionInfo(PlotWidgetTestCase):
    """Tests for PositionInfo widget."""

    def _createPlot(self):
        return PlotWindow()

    def setUp(self):
        super(TestPositionInfo, self).setUp()
        self.mouseMove(self.plot, pos=(0, 0))
        self.qapp.processEvents()
        self.qWait(100)

    def tearDown(self):
        super(TestPositionInfo, self).tearDown()

    def _test(self, positionWidget, converterNames, **kwargs):
        """General test of PositionInfo.

        - Add it to a toolbar and
        - Move mouse around the center of the PlotWindow.
        """
        toolBar = qt.QToolBar()
        self.plot.addToolBar(qt.Qt.BottomToolBarArea, toolBar)

        toolBar.addWidget(positionWidget)

        converters = positionWidget.getConverters()
        self.assertEqual(len(converters), len(converterNames))
        for index, name in enumerate(converterNames):
            self.assertEqual(converters[index][0], name)

        with TestLogging(tools.__name__, **kwargs):
            # Move mouse to center
            center = self.plot.size() / 2
            self.mouseMove(self.plot, pos=(center.width(), center.height()))
            # Move out
            self.mouseMove(self.plot, pos=(1, 1))

    def testDefaultConverters(self):
        """Test PositionInfo with default converters"""
        positionWidget = tools.PositionInfo(plot=self.plot)
        self._test(positionWidget, ('X', 'Y'))

    def testCustomConverters(self):
        """Test PositionInfo with custom converters"""
        converters = [
            ('Coords', lambda x, y: (int(x), int(y))),
            ('Radius', lambda x, y: numpy.sqrt(x * x + y * y)),
            ('Angle', lambda x, y: numpy.degrees(numpy.arctan2(y, x)))
        ]
        positionWidget = tools.PositionInfo(plot=self.plot,
                                            converters=converters)
        self._test(positionWidget, ('Coords', 'Radius', 'Angle'))

    def testFailingConverters(self):
        """Test PositionInfo with failing custom converters"""
        def raiseException(x, y):
            raise RuntimeError()

        positionWidget = tools.PositionInfo(
            plot=self.plot,
            converters=[('Exception', raiseException)])
        self._test(positionWidget, ['Exception'], error=2)

    def testUpdate(self):
        """Test :meth:`PositionInfo.updateInfo`"""
        calls = []

        def update(calls, x, y):  # Get number of calls
            calls.append((x, y))
            return len(calls)

        positionWidget = tools.PositionInfo(
            plot=self.plot,
            converters=[('Call count', functools.partial(update, calls))])

        positionWidget.updateInfo()
        self.assertEqual(len(calls), 1)


class TestPlotToolsToolbars(PlotWidgetTestCase):
    """Tests toolbars from silx.gui.plot.tools"""

    def test(self):
        """"Add all toolbars"""
        for tbClass in (tools.InteractiveModeToolBar,
                        tools.ImageToolBar,
                        tools.CurveToolBar,
                        tools.OutputToolBar):
            tb = tbClass(parent=self.plot, plot=self.plot)
        self.plot.addToolBar(tb)


def suite():
    test_suite = unittest.TestSuite()
    # test_suite.addTest(positionInfoTestSuite)
    for testClass in (TestPositionInfo, TestPlotToolsToolbars):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(
            testClass))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Tests for html module"""

__authors__ = ["V. Valls"]
__license__ = "MIT"
__date__ = "17/01/2018"


import unittest
from .. import deprecation
from silx.utils import testutils


class TestDeprecation(unittest.TestCase):
    """Tests for deprecation module."""

    @deprecation.deprecated
    def deprecatedWithoutParam(self):
        pass

    @deprecation.deprecated(reason="r", replacement="r", since_version="v")
    def deprecatedWithParams(self):
        pass

    @deprecation.deprecated(reason="r", replacement="r", since_version="v", only_once=True)
    def deprecatedOnlyOnce(self):
        pass

    @deprecation.deprecated(reason="r", replacement="r", since_version="v", only_once=False)
    def deprecatedEveryTime(self):
        pass

    @testutils.test_logging(deprecation.depreclog.name, warning=1)
    def testAnnotationWithoutParam(self):
        self.deprecatedWithoutParam()

    @testutils.test_logging(deprecation.depreclog.name, warning=1)
    def testAnnotationWithParams(self):
        self.deprecatedWithParams()

    @testutils.test_logging(deprecation.depreclog.name, warning=3)
    def testLoggedEveryTime(self):
        """Logged everytime cause it is 3 different locations"""
        self.deprecatedOnlyOnce()
        self.deprecatedOnlyOnce()
        self.deprecatedOnlyOnce()

    @testutils.test_logging(deprecation.depreclog.name, warning=1)
    def testLoggedSingleTime(self):
        def log():
            self.deprecatedOnlyOnce()
        log()
        log()
        log()

    @testutils.test_logging(deprecation.depreclog.name, warning=3)
    def testLoggedEveryTime2(self):
        self.deprecatedEveryTime()
        self.deprecatedEveryTime()
        self.deprecatedEveryTime()

    @testutils.test_logging(deprecation.depreclog.name, warning=1)
    def testWarning(self):
        deprecation.deprecated_warning(type_="t", name="n")

    def testBacktrace(self):
        testLogging = testutils.TestLogging(deprecation.depreclog.name)
        with testLogging:
            self.deprecatedEveryTime()
        message = testLogging.records[0].getMessage()
        filename = __file__.replace(".pyc", ".py")
        self.assertTrue(filename in message)
        self.assertTrue("testBacktrace" in message)


def suite():
    test_suite = unittest.TestSuite()
    loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
    test_suite.addTest(loadTests(TestDeprecation))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Matplotlib Plot backend."""

from __future__ import division

__authors__ = ["V.A. Sole", "T. Vincent, H. Payno"]
__license__ = "MIT"
__date__ = "18/10/2017"


import logging
import datetime as dt
import numpy

from pkg_resources import parse_version as _parse_version


_logger = logging.getLogger(__name__)


from ... import qt

# First of all init matplotlib and set its backend
from ..matplotlib import FigureCanvasQTAgg
import matplotlib
from matplotlib.container import Container
from matplotlib.figure import Figure
from matplotlib.patches import Rectangle, Polygon
from matplotlib.image import AxesImage
from matplotlib.backend_bases import MouseEvent
from matplotlib.lines import Line2D
from matplotlib.collections import PathCollection, LineCollection
from matplotlib.ticker import Formatter, ScalarFormatter, Locator



from ..matplotlib.ModestImage import ModestImage
from . import BackendBase
from .._utils import FLOAT32_MINPOS
from .._utils.dtime_ticklayout import calcTicks, bestFormatString, timestamp



class NiceDateLocator(Locator):
    """
    Matplotlib Locator that uses Nice Numbers algorithm (adapted to dates)
    to find the tick locations. This results in the same number behaviour
    as when using the silx Open GL backend.

    Expects the data to be posix timestampes (i.e. seconds since 1970)
    """
    def __init__(self, numTicks=5, tz=None):
        """
        :param numTicks: target number of ticks
        :param datetime.tzinfo tz: optional time zone. None is local time.
        """
        super(NiceDateLocator, self).__init__()
        self.numTicks = numTicks

        self._spacing = None
        self._unit = None
        self.tz = tz

    @property
    def spacing(self):
        """ The current spacing. Will be updated when new tick value are made"""
        return self._spacing

    @property
    def unit(self):
        """ The current DtUnit. Will be updated when new tick value are made"""
        return self._unit

    def __call__(self):
        """Return the locations of the ticks"""
        vmin, vmax = self.axis.get_view_interval()
        return self.tick_values(vmin, vmax)

    def tick_values(self, vmin, vmax):
        """ Calculates tick values
        """
        if vmax < vmin:
            vmin, vmax = vmax, vmin

        # vmin and vmax should be timestamps (i.e. seconds since 1 Jan 1970)
        dtMin = dt.datetime.fromtimestamp(vmin, tz=self.tz)
        dtMax = dt.datetime.fromtimestamp(vmax, tz=self.tz)
        dtTicks, self._spacing, self._unit = \
            calcTicks(dtMin, dtMax, self.numTicks)

        # Convert datetime back to time stamps.
        ticks = [timestamp(dtTick) for dtTick in dtTicks]
        return ticks



class NiceAutoDateFormatter(Formatter):
    """
    Matplotlib FuncFormatter that is linked to a NiceDateLocator and gives the
    best possible formats given the locators current spacing an date unit.
    """

    def __init__(self, locator, tz=None):
        """
        :param niceDateLocator: a NiceDateLocator object
        :param datetime.tzinfo tz: optional time zone. None is local time.
        """
        super(NiceAutoDateFormatter, self).__init__()
        self.locator = locator
        self.tz = tz

    @property
    def formatString(self):
        if self.locator.spacing is None or self.locator.unit is None:
            # Locator has no spacing or units yet. Return elaborate fmtString
            return "Y-%m-%d %H:%M:%S"
        else:
            return bestFormatString(self.locator.spacing, self.locator.unit)


    def __call__(self, x, pos=None):
        """Return the format for tick val *x* at position *pos*
           Expects x to be a POSIX timestamp (seconds since 1 Jan 1970)
        """
        dateTime = dt.datetime.fromtimestamp(x, tz=self.tz)
        tickStr = dateTime.strftime(self.formatString)
        return tickStr




class _MarkerContainer(Container):
    """Marker artists container supporting draw/remove and text position update

    :param artists:
        Iterable with either one Line2D or a Line2D and a Text.
        The use of an iterable if enforced by Container being
        a subclass of tuple that defines a specific __new__.
    :param x: X coordinate of the marker (None for horizontal lines)
    :param y: Y coordinate of the marker (None for vertical lines)
    """

    def __init__(self, artists, x, y):
        self.line = artists[0]
        self.text = artists[1] if len(artists) > 1 else None
        self.x = x
        self.y = y

        Container.__init__(self, artists)

    def draw(self, *args, **kwargs):
        """artist-like draw to broadcast draw to line and text"""
        self.line.draw(*args, **kwargs)
        if self.text is not None:
            self.text.draw(*args, **kwargs)

    def updateMarkerText(self, xmin, xmax, ymin, ymax):
        """Update marker text position and visibility according to plot limits

        :param xmin: X axis lower limit
        :param xmax: X axis upper limit
        :param ymin: Y axis lower limit
        :param ymax: Y axis upprt limit
        """
        if self.text is not None:
            visible = ((self.x is None or xmin <= self.x <= xmax) and
                       (self.y is None or ymin <= self.y <= ymax))
            self.text.set_visible(visible)

            if self.x is not None and self.y is None:  # vertical line
                delta = abs(ymax - ymin)
                if ymin > ymax:
                    ymax = ymin
                ymax -= 0.005 * delta
                self.text.set_y(ymax)

            if self.x is None and self.y is not None:  # Horizontal line
                delta = abs(xmax - xmin)
                if xmin > xmax:
                    xmax = xmin
                xmax -= 0.005 * delta
                self.text.set_x(xmax)


class BackendMatplotlib(BackendBase.BackendBase):
    """Base class for Matplotlib backend without a FigureCanvas.

    For interactive on screen plot, see :class:`BackendMatplotlibQt`.

    See :class:`BackendBase.BackendBase` for public API documentation.
    """

    def __init__(self, plot, parent=None):
        super(BackendMatplotlib, self).__init__(plot, parent)

        # matplotlib is handling keep aspect ratio at draw time
        # When keep aspect ratio is on, and one changes the limits and
        # ask them *before* next draw has been performed he will get the
        # limits without applying keep aspect ratio.
        # This attribute is used to ensure consistent values returned
        # when getting the limits at the expense of a replot
        self._dirtyLimits = True
        self._axesDisplayed = True
        self._matplotlibVersion = _parse_version(matplotlib.__version__)

        self.fig = Figure()
        self.fig.set_facecolor("w")

        self.ax = self.fig.add_axes([.15, .15, .75, .75], label="left")
        self.ax2 = self.ax.twinx()
        self.ax2.set_label("right")

        # disable the use of offsets
        try:
            self.ax.get_yaxis().get_major_formatter().set_useOffset(False)
            self.ax.get_xaxis().get_major_formatter().set_useOffset(False)
            self.ax2.get_yaxis().get_major_formatter().set_useOffset(False)
            self.ax2.get_xaxis().get_major_formatter().set_useOffset(False)
        except:
            _logger.warning('Cannot disabled axes offsets in %s ' \
                            % matplotlib.__version__)
        
        # critical for picking!!!!
        self.ax2.set_zorder(0)
        self.ax2.set_autoscaley_on(True)
        self.ax.set_zorder(1)
        # this works but the figure color is left
        if self._matplotlibVersion < _parse_version('2'):
            self.ax.set_axis_bgcolor('none')
        else:
            self.ax.set_facecolor('none')
        self.fig.sca(self.ax)

        self._overlays = set()
        self._background = None

        self._colormaps = {}

        self._graphCursor = tuple()

        self._enableAxis('right', False)
        self._isXAxisTimeSeries = False

    # Add methods

    def addCurve(self, x, y, legend,
                 color, symbol, linewidth, linestyle,
                 yaxis,
                 xerror, yerror, z, selectable,
                 fill, alpha, symbolsize):
        for parameter in (x, y, legend, color, symbol, linewidth, linestyle,
                          yaxis, z, selectable, fill, alpha, symbolsize):
            assert parameter is not None
        assert yaxis in ('left', 'right')

        if (len(color) == 4 and
                type(color[3]) in [type(1), numpy.uint8, numpy.int8]):
            color = numpy.array(color, dtype=numpy.float) / 255.

        if yaxis == "right":
            axes = self.ax2
            self._enableAxis("right", True)
        else:
            axes = self.ax

        picker = 3 if selectable else None

        artists = []  # All the artists composing the curve

        # First add errorbars if any so they are behind the curve
        if xerror is not None or yerror is not None:
            if hasattr(color, 'dtype') and len(color) == len(x):
                errorbarColor = 'k'
            else:
                errorbarColor = color

            # On Debian 7 at least, Nx1 array yerr does not seems supported
            if (isinstance(yerror, numpy.ndarray) and yerror.ndim == 2 and
                    yerror.shape[1] == 1 and len(x) != 1):
                yerror = numpy.ravel(yerror)

            errorbars = axes.errorbar(x, y, label=legend,
                                      xerr=xerror, yerr=yerror,
                                      linestyle=' ', color=errorbarColor)
            artists += list(errorbars.get_children())

        if hasattr(color, 'dtype') and len(color) == len(x):
            # scatter plot
            if color.dtype not in [numpy.float32, numpy.float]:
                actualColor = color / 255.
            else:
                actualColor = color

            if linestyle not in ["", " ", None]:
                # scatter plot with an actual line ...
                # we need to assign a color ...
                curveList = axes.plot(x, y, label=legend,
                                      linestyle=linestyle,
                                      color=actualColor[0],
                                      linewidth=linewidth,
                                      picker=picker,
                                      marker=None)
                artists += list(curveList)

            scatter = axes.scatter(x, y,
                                   label=legend,
                                   color=actualColor,
                                   marker=symbol,
                                   picker=picker,
                                   s=symbolsize**2)
            artists.append(scatter)

            if fill:
                artists.append(axes.fill_between(
                    x, FLOAT32_MINPOS, y, facecolor=actualColor[0], linestyle=''))

        else:  # Curve
            curveList = axes.plot(x, y,
                                  label=legend,
                                  linestyle=linestyle,
                                  color=color,
                                  linewidth=linewidth,
                                  marker=symbol,
                                  picker=picker,
                                  markersize=symbolsize)
            artists += list(curveList)

            if fill:
                artists.append(
                    axes.fill_between(x, FLOAT32_MINPOS, y, facecolor=color))

        for artist in artists:
            artist.set_zorder(z)
            if alpha < 1:
                artist.set_alpha(alpha)

        return Container(artists)

    def addImage(self, data, legend,
                 origin, scale, z,
                 selectable, draggable,
                 colormap, alpha):
        # Non-uniform image
        # http://wiki.scipy.org/Cookbook/Histograms
        # Non-linear axes
        # http://stackoverflow.com/questions/11488800/non-linear-axes-for-imshow-in-matplotlib
        for parameter in (data, legend, origin, scale, z,
                          selectable, draggable):
            assert parameter is not None

        origin = float(origin[0]), float(origin[1])
        scale = float(scale[0]), float(scale[1])
        height, width = data.shape[0:2]

        picker = (selectable or draggable)

        # Debian 7 specific support
        # No transparent colormap with matplotlib < 1.2.0
        # Add support for transparent colormap for uint8 data with
        # colormap with 256 colors, linear norm, [0, 255] range
        if self._matplotlibVersion < _parse_version('1.2.0'):
            if (len(data.shape) == 2 and colormap.getName() is None and
                    colormap.getColormapLUT() is not None):
                colors = colormap.getColormapLUT()
                if (colors.shape[-1] == 4 and
                        not numpy.all(numpy.equal(colors[3], 255))):
                    # This is a transparent colormap
                    if (colors.shape == (256, 4) and
                            colormap.getNormalization() == 'linear' and
                            not colormap.isAutoscale() and
                            colormap.getVMin() == 0 and
                            colormap.getVMax() == 255 and
                            data.dtype == numpy.uint8):
                        # Supported case, convert data to RGBA
                        data = colors[data.reshape(-1)].reshape(
                            data.shape + (4,))
                    else:
                        _logger.warning(
                            'matplotlib %s does not support transparent '
                            'colormap.', matplotlib.__version__)

        if ((height * width) > 5.0e5 and
                origin == (0., 0.) and scale == (1., 1.)):
            imageClass = ModestImage
        else:
            imageClass = AxesImage

        # All image are shown as RGBA image
        image = imageClass(self.ax,
                           label="__IMAGE__" + legend,
                           interpolation='nearest',
                           picker=picker,
                           zorder=z,
                           origin='lower')

        if alpha < 1:
            image.set_alpha(alpha)

        # Set image extent
        xmin = origin[0]
        xmax = xmin + scale[0] * width
        if scale[0] < 0.:
            xmin, xmax = xmax, xmin

        ymin = origin[1]
        ymax = ymin + scale[1] * height
        if scale[1] < 0.:
            ymin, ymax = ymax, ymin

        image.set_extent((xmin, xmax, ymin, ymax))

        # Set image data
        if scale[0] < 0. or scale[1] < 0.:
            # For negative scale, step by -1
            xstep = 1 if scale[0] >= 0. else -1
            ystep = 1 if scale[1] >= 0. else -1
            data = data[::ystep, ::xstep]

        if self._matplotlibVersion < _parse_version('2.1'):
            # matplotlib 1.4.2 do not support float128
            dtype = data.dtype
            if dtype.kind == "f" and dtype.itemsize >= 16:
                _logger.warning("Your matplotlib version do not support "
                                "float128. Data converted to float64.")
                data = data.astype(numpy.float64)

        if data.ndim == 2:  # Data image, convert to RGBA image
            data = colormap.applyToData(data)

        image.set_data(data)

        self.ax.add_artist(image)

        return image

    def addItem(self, x, y, legend, shape, color, fill, overlay, z):
        xView = numpy.array(x, copy=False)
        yView = numpy.array(y, copy=False)

        if shape == "line":
            item = self.ax.plot(x, y, label=legend, color=color,
                                linestyle='-', marker=None)[0]

        elif shape == "hline":
            if hasattr(y, "__len__"):
                y = y[-1]
            item = self.ax.axhline(y, label=legend, color=color)

        elif shape == "vline":
            if hasattr(x, "__len__"):
                x = x[-1]
            item = self.ax.axvline(x, label=legend, color=color)

        elif shape == 'rectangle':
            xMin = numpy.nanmin(xView)
            xMax = numpy.nanmax(xView)
            yMin = numpy.nanmin(yView)
            yMax = numpy.nanmax(yView)
            w = xMax - xMin
            h = yMax - yMin
            item = Rectangle(xy=(xMin, yMin),
                             width=w,
                             height=h,
                             fill=False,
                             color=color)
            if fill:
                item.set_hatch('.')

            self.ax.add_patch(item)

        elif shape in ('polygon', 'polylines'):
            points = numpy.array((xView, yView)).T
            if shape == 'polygon':
                closed = True
            else:  # shape == 'polylines'
                closed = numpy.all(numpy.equal(points[0], points[-1]))
            item = Polygon(points,
                           closed=closed,
                           fill=False,
                           label=legend,
                           color=color)
            if fill and shape == 'polygon':
                item.set_hatch('/')

            self.ax.add_patch(item)

        else:
            raise NotImplementedError("Unsupported item shape %s" % shape)

        item.set_zorder(z)

        if overlay:
            item.set_animated(True)
            self._overlays.add(item)

        return item

    def addMarker(self, x, y, legend, text, color,
                  selectable, draggable,
                  symbol, constraint):
        legend = "__MARKER__" + legend

        textArtist = None

        xmin, xmax = self.getGraphXLimits()
        ymin, ymax = self.getGraphYLimits(axis='left')

        if x is not None and y is not None:
            line = self.ax.plot(x, y, label=legend,
                                linestyle=" ",
                                color=color,
                                marker=symbol,
                                markersize=10.)[-1]

            if text is not None:
                if symbol is None:
                    valign = 'baseline'
                else:
                    valign = 'top'
                    text = "  " + text

                textArtist = self.ax.text(x, y, text,
                                          color=color,
                                          horizontalalignment='left',
                                          verticalalignment=valign)

        elif x is not None:
            line = self.ax.axvline(x, label=legend, color=color)
            if text is not None:
                # Y position will be updated in updateMarkerText call
                textArtist = self.ax.text(x, 1., " " + text,
                                          color=color,
                                          horizontalalignment='left',
                                          verticalalignment='top')

        elif y is not None:
            line = self.ax.axhline(y, label=legend, color=color)

            if text is not None:
                # X position will be updated in updateMarkerText call
                textArtist = self.ax.text(1., y, " " + text,
                                          color=color,
                                          horizontalalignment='right',
                                          verticalalignment='top')

        else:
            raise RuntimeError('A marker must at least have one coordinate')

        if selectable or draggable:
            line.set_picker(5)

        # All markers are overlays
        line.set_animated(True)
        if textArtist is not None:
            textArtist.set_animated(True)

        artists = [line] if textArtist is None else [line, textArtist]
        container = _MarkerContainer(artists, x, y)
        container.updateMarkerText(xmin, xmax, ymin, ymax)
        self._overlays.add(container)

        return container

    def _updateMarkers(self):
        xmin, xmax = self.ax.get_xbound()
        ymin, ymax = self.ax.get_ybound()
        for item in self._overlays:
            if isinstance(item, _MarkerContainer):
                item.updateMarkerText(xmin, xmax, ymin, ymax)

    # Remove methods

    def remove(self, item):
        # Warning: It also needs to remove extra stuff if added as for markers
        self._overlays.discard(item)
        try:
            item.remove()
        except ValueError:
            pass  # Already removed e.g., in set[X|Y]AxisLogarithmic

    # Interaction methods

    def setGraphCursor(self, flag, color, linewidth, linestyle):
        if flag:
            lineh = self.ax.axhline(
                self.ax.get_ybound()[0], visible=False, color=color,
                linewidth=linewidth, linestyle=linestyle)
            lineh.set_animated(True)

            linev = self.ax.axvline(
                self.ax.get_xbound()[0], visible=False, color=color,
                linewidth=linewidth, linestyle=linestyle)
            linev.set_animated(True)

            self._graphCursor = lineh, linev
        else:
            if self._graphCursor is not None:
                lineh, linev = self._graphCursor
                lineh.remove()
                linev.remove()
                self._graphCursor = tuple()

    # Active curve

    def setCurveColor(self, curve, color):
        # Store Line2D and PathCollection
        for artist in curve.get_children():
            if isinstance(artist, (Line2D, LineCollection)):
                artist.set_color(color)
            elif isinstance(artist, PathCollection):
                artist.set_facecolors(color)
                artist.set_edgecolors(color)
            else:
                _logger.warning(
                    'setActiveCurve ignoring artist %s', str(artist))

    # Misc.

    def getWidgetHandle(self):
        return self.fig.canvas

    def _enableAxis(self, axis, flag=True):
        """Show/hide Y axis

        :param str axis: Axis name: 'left' or 'right'
        :param bool flag: Default, True
        """
        assert axis in ('right', 'left')
        axes = self.ax2 if axis == 'right' else self.ax
        axes.get_yaxis().set_visible(flag)

    def replot(self):
        """Do not perform rendering.

        Override in subclass to actually draw something.
        """
        # TODO images, markers? scatter plot? move in remove?
        # Right Y axis only support curve for now
        # Hide right Y axis if no line is present
        self._dirtyLimits = False
        if not self.ax2.lines:
            self._enableAxis('right', False)

    def saveGraph(self, fileName, fileFormat, dpi):
        # fileName can be also a StringIO or file instance
        if dpi is not None:
            self.fig.savefig(fileName, format=fileFormat, dpi=dpi)
        else:
            self.fig.savefig(fileName, format=fileFormat)
        self._plot._setDirtyPlot()

    # Graph labels

    def setGraphTitle(self, title):
        self.ax.set_title(title)

    def setGraphXLabel(self, label):
        self.ax.set_xlabel(label)

    def setGraphYLabel(self, label, axis):
        axes = self.ax if axis == 'left' else self.ax2
        axes.set_ylabel(label)

    # Graph limits

    def setLimits(self, xmin, xmax, ymin, ymax, y2min=None, y2max=None):
        # Let matplotlib taking care of keep aspect ratio if any
        self._dirtyLimits = True
        self.ax.set_xlim(min(xmin, xmax), max(xmin, xmax))

        if y2min is not None and y2max is not None:
            if not self.isYAxisInverted():
                self.ax2.set_ylim(min(y2min, y2max), max(y2min, y2max))
            else:
                self.ax2.set_ylim(max(y2min, y2max), min(y2min, y2max))

        if not self.isYAxisInverted():
            self.ax.set_ylim(min(ymin, ymax), max(ymin, ymax))
        else:
            self.ax.set_ylim(max(ymin, ymax), min(ymin, ymax))

        self._updateMarkers()

    def getGraphXLimits(self):
        if self._dirtyLimits and self.isKeepDataAspectRatio():
            self.replot()  # makes sure we get the right limits
        return self.ax.get_xbound()

    def setGraphXLimits(self, xmin, xmax):
        self._dirtyLimits = True
        self.ax.set_xlim(min(xmin, xmax), max(xmin, xmax))
        self._updateMarkers()

    def getGraphYLimits(self, axis):
        assert axis in ('left', 'right')
        ax = self.ax2 if axis == 'right' else self.ax

        if not ax.get_visible():
            return None

        if self._dirtyLimits and self.isKeepDataAspectRatio():
            self.replot()  # makes sure we get the right limits

        return ax.get_ybound()

    def setGraphYLimits(self, ymin, ymax, axis):
        ax = self.ax2 if axis == 'right' else self.ax
        if ymax < ymin:
            ymin, ymax = ymax, ymin
        self._dirtyLimits = True

        if self.isKeepDataAspectRatio():
            # matplotlib keeps limits of shared axis when keeping aspect ratio
            # So x limits are kept when changing y limits....
            # Change x limits first by taking into account aspect ratio
            # and then change y limits.. so matplotlib does not need
            # to make change (to y) to keep aspect ratio
            xmin, xmax = ax.get_xbound()
            curYMin, curYMax = ax.get_ybound()

            newXRange = (xmax - xmin) * (ymax - ymin) / (curYMax - curYMin)
            xcenter = 0.5 * (xmin + xmax)
            ax.set_xlim(xcenter - 0.5 * newXRange, xcenter + 0.5 * newXRange)

        if not self.isYAxisInverted():
            ax.set_ylim(ymin, ymax)
        else:
            ax.set_ylim(ymax, ymin)

        self._updateMarkers()

    # Graph axes

    def setXAxisTimeZone(self, tz):
        super(BackendMatplotlib, self).setXAxisTimeZone(tz)

        # Make new formatter and locator with the time zone.
        self.setXAxisTimeSeries(self.isXAxisTimeSeries())

    def isXAxisTimeSeries(self):
        return self._isXAxisTimeSeries

    def setXAxisTimeSeries(self, isTimeSeries):
        self._isXAxisTimeSeries = isTimeSeries
        if self._isXAxisTimeSeries:
            # We can't use a matplotlib.dates.DateFormatter because it expects
            # the data to be in datetimes. Silx works internally with
            # timestamps (floats).
            locator = NiceDateLocator(tz=self.getXAxisTimeZone())
            self.ax.xaxis.set_major_locator(locator)
            self.ax.xaxis.set_major_formatter(
                NiceAutoDateFormatter(locator, tz=self.getXAxisTimeZone()))
        else:
            try:
                scalarFormatter = ScalarFormatter(useOffset=False)
            except:
                _logger.warning('Cannot disabled axes offsets in %s ' %
                                matplotlib.__version__)
                scalarFormatter = ScalarFormatter()
            self.ax.xaxis.set_major_formatter(scalarFormatter)

    def setXAxisLogarithmic(self, flag):
        # Workaround for matplotlib 2.1.0 when one tries to set an axis
        # to log scale with both limits <= 0
        # In this case a draw with positive limits is needed first
        if flag and self._matplotlibVersion >= _parse_version('2.1.0'):
            xlim = self.ax.get_xlim()
            if xlim[0] <= 0 and xlim[1] <= 0:
                self.ax.set_xlim(1, 10)
                self.draw()

        self.ax2.set_xscale('log' if flag else 'linear')
        self.ax.set_xscale('log' if flag else 'linear')

    def setYAxisLogarithmic(self, flag):
        # Workaround for matplotlib 2.0 issue with negative bounds
        # before switching to log scale
        if flag and self._matplotlibVersion >= _parse_version('2.0.0'):
            redraw = False
            for axis, dataRangeIndex in ((self.ax, 1), (self.ax2, 2)):
                ylim = axis.get_ylim()
                if ylim[0] <= 0 or ylim[1] <= 0:
                    dataRange = self._plot.getDataRange()[dataRangeIndex]
                    if dataRange is None:
                        dataRange = 1, 100  # Fallback
                    axis.set_ylim(*dataRange)
                    redraw = True
            if redraw:
                self.draw()

        self.ax2.set_yscale('log' if flag else 'linear')
        self.ax.set_yscale('log' if flag else 'linear')

    def setYAxisInverted(self, flag):
        if self.ax.yaxis_inverted() != bool(flag):
            self.ax.invert_yaxis()

    def isYAxisInverted(self):
        return self.ax.yaxis_inverted()

    def isKeepDataAspectRatio(self):
        return self.ax.get_aspect() in (1.0, 'equal')

    def setKeepDataAspectRatio(self, flag):
        self.ax.set_aspect(1.0 if flag else 'auto')
        self.ax2.set_aspect(1.0 if flag else 'auto')

    def setGraphGrid(self, which):
        self.ax.grid(False, which='both')  # Disable all grid first
        if which is not None:
            self.ax.grid(True, which=which)

    # Data <-> Pixel coordinates conversion

    def _mplQtYAxisCoordConversion(self, y):
        """Qt origin (top) to/from matplotlib origin (bottom) conversion.

        :rtype: float
        """
        height = self.fig.get_window_extent().height
        return height - y

    def dataToPixel(self, x, y, axis):
        ax = self.ax2 if axis == "right" else self.ax

        pixels = ax.transData.transform_point((x, y))
        xPixel, yPixel = pixels.T

        # Convert from matplotlib origin (bottom) to Qt origin (top)
        yPixel = self._mplQtYAxisCoordConversion(yPixel)

        return xPixel, yPixel

    def pixelToData(self, x, y, axis, check):
        ax = self.ax2 if axis == "right" else self.ax

        # Convert from Qt origin (top) to matplotlib origin (bottom)
        y = self._mplQtYAxisCoordConversion(y)

        inv = ax.transData.inverted()
        x, y = inv.transform_point((x, y))

        if check:
            xmin, xmax = self.getGraphXLimits()
            ymin, ymax = self.getGraphYLimits(axis=axis)

            if x > xmax or x < xmin or y > ymax or y < ymin:
                return None  # (x, y) is out of plot area

        return x, y

    def getPlotBoundsInPixels(self):
        bbox = self.ax.get_window_extent()
        # Warning this is not returning int...
        return (bbox.xmin,
                self._mplQtYAxisCoordConversion(bbox.ymax),
                bbox.width,
                bbox.height)

    def setAxesDisplayed(self, displayed):
        """Display or not the axes.

        :param bool displayed: If `True` axes are displayed. If `False` axes
            are not anymore visible and the margin used for them is removed.
        """
        BackendBase.BackendBase.setAxesDisplayed(self, displayed)
        if displayed:
            # show axes and viewbox rect
            self.ax.set_axis_on()
            self.ax2.set_axis_on()
            # set the default margins
            self.ax.set_position([.15, .15, .75, .75])
            self.ax2.set_position([.15, .15, .75, .75])
        else:
            # hide axes and viewbox rect
            self.ax.set_axis_off()
            self.ax2.set_axis_off()
            # remove external margins
            self.ax.set_position([0, 0, 1, 1])
            self.ax2.set_position([0, 0, 1, 1])
        self._plot._setDirtyPlot()


class BackendMatplotlibQt(FigureCanvasQTAgg, BackendMatplotlib):
    """QWidget matplotlib backend using a QtAgg canvas.

    It adds fast overlay drawing and mouse event management.
    """

    _sigPostRedisplay = qt.Signal()
    """Signal handling automatic asynchronous replot"""

    def __init__(self, plot, parent=None):
        BackendMatplotlib.__init__(self, plot, parent)
        FigureCanvasQTAgg.__init__(self, self.fig)
        self.setParent(parent)

        self._limitsBeforeResize = None

        FigureCanvasQTAgg.setSizePolicy(
            self, qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding)
        FigureCanvasQTAgg.updateGeometry(self)

        # Make postRedisplay asynchronous using Qt signal
        self._sigPostRedisplay.connect(
            super(BackendMatplotlibQt, self).postRedisplay,
            qt.Qt.QueuedConnection)

        self._picked = None

        self.mpl_connect('button_press_event', self._onMousePress)
        self.mpl_connect('button_release_event', self._onMouseRelease)
        self.mpl_connect('motion_notify_event', self._onMouseMove)
        self.mpl_connect('scroll_event', self._onMouseWheel)

    def contextMenuEvent(self, event):
        """Override QWidget.contextMenuEvent to implement the context menu"""
        # Makes sure it is overridden (issue with PySide)
        BackendBase.BackendBase.contextMenuEvent(self, event)

    def postRedisplay(self):
        self._sigPostRedisplay.emit()

    # Mouse event forwarding

    _MPL_TO_PLOT_BUTTONS = {1: 'left', 2: 'middle', 3: 'right'}

    def _onMousePress(self, event):
        self._plot.onMousePress(
            event.x, self._mplQtYAxisCoordConversion(event.y),
            self._MPL_TO_PLOT_BUTTONS[event.button])

    def _onMouseMove(self, event):
        if self._graphCursor:
            lineh, linev = self._graphCursor
            if event.inaxes != self.ax and lineh.get_visible():
                lineh.set_visible(False)
                linev.set_visible(False)
                self._plot._setDirtyPlot(overlayOnly=True)
            else:
                linev.set_visible(True)
                linev.set_xdata((event.xdata, event.xdata))
                lineh.set_visible(True)
                lineh.set_ydata((event.ydata, event.ydata))
                self._plot._setDirtyPlot(overlayOnly=True)
            # onMouseMove must trigger replot if dirty flag is raised

        self._plot.onMouseMove(
            event.x, self._mplQtYAxisCoordConversion(event.y))

    def _onMouseRelease(self, event):
        self._plot.onMouseRelease(
            event.x, self._mplQtYAxisCoordConversion(event.y),
            self._MPL_TO_PLOT_BUTTONS[event.button])

    def _onMouseWheel(self, event):
        self._plot.onMouseWheel(
            event.x, self._mplQtYAxisCoordConversion(event.y), event.step)

    def leaveEvent(self, event):
        """QWidget event handler"""
        self._plot.onMouseLeaveWidget()

    # picking

    def _onPick(self, event):
        # TODO not very nice and fragile, find a better way?
        # Make a selection according to kind
        if self._picked is None:
            _logger.error('Internal picking error')
            return

        label = event.artist.get_label()
        if label.startswith('__MARKER__'):
            self._picked.append({'kind': 'marker', 'legend': label[10:]})

        elif label.startswith('__IMAGE__'):
            self._picked.append({'kind': 'image', 'legend': label[9:]})

        else:  # it's a curve, item have no picker for now
            if not isinstance(event.artist, (PathCollection, Line2D)):
                _logger.info('Unsupported artist, ignored')
                return

            self._picked.append({'kind': 'curve', 'legend': label,
                                 'indices': event.ind})

    def pickItems(self, x, y, kinds):
        self._picked = []

        # Weird way to do an explicit picking: Simulate a button press event
        mouseEvent = MouseEvent('button_press_event',
                                self, x, self._mplQtYAxisCoordConversion(y))
        cid = self.mpl_connect('pick_event', self._onPick)
        self.fig.pick(mouseEvent)
        self.mpl_disconnect(cid)

        picked = [p for p in self._picked if p['kind'] in kinds]
        self._picked = None

        return picked

    # replot control

    def resizeEvent(self, event):
        # Store current limits
        self._limitsBeforeResize = (
            self.ax.get_xbound(), self.ax.get_ybound(), self.ax2.get_ybound())

        FigureCanvasQTAgg.resizeEvent(self, event)
        if self.isKeepDataAspectRatio() or self._overlays or self._graphCursor:
            # This is needed with matplotlib 1.5.x and 2.0.x
            self._plot._setDirtyPlot()

    def _drawOverlays(self):
        """Draw overlays if any."""
        if self._overlays or self._graphCursor:
            # There is some overlays or crosshair

            # This assume that items are only on left/bottom Axes
            for item in self._overlays:
                self.ax.draw_artist(item)

            for item in self._graphCursor:
                self.ax.draw_artist(item)

    def draw(self):
        """Overload draw

        It performs a full redraw (including overlays) of the plot.
        It also resets background and emit limits changed signal.

        This is directly called by matplotlib for widget resize.
        """
        # Starting with mpl 2.1.0, toggling autoscale raises a ValueError
        # in some situations. See #1081, #1136, #1163,
        if self._matplotlibVersion >= _parse_version("2.0.0"):
            try:
                FigureCanvasQTAgg.draw(self)
            except ValueError as err:
                _logger.debug(
                    "ValueError caught while calling FigureCanvasQTAgg.draw: "
                    "'%s'", err)
        else:
            FigureCanvasQTAgg.draw(self)

        if self._overlays or self._graphCursor:
            # Save background
            self._background = self.copy_from_bbox(self.fig.bbox)
        else:
            self._background = None  # Reset background

        # Check if limits changed due to a resize of the widget
        if self._limitsBeforeResize is not None:
            xLimits, yLimits, yRightLimits = self._limitsBeforeResize
            self._limitsBeforeResize = None

            if (xLimits != self.ax.get_xbound() or
                    yLimits != self.ax.get_ybound()):
                self._updateMarkers()

            if xLimits != self.ax.get_xbound():
                self._plot.getXAxis()._emitLimitsChanged()
            if yLimits != self.ax.get_ybound():
                self._plot.getYAxis(axis='left')._emitLimitsChanged()
            if yRightLimits != self.ax2.get_ybound():
                self._plot.getYAxis(axis='right')._emitLimitsChanged()

        self._drawOverlays()

    def replot(self):
        BackendMatplotlib.replot(self)

        dirtyFlag = self._plot._getDirtyPlot()

        if dirtyFlag == 'overlay':
            # Only redraw overlays using fast rendering path
            if self._background is None:
                self._background = self.copy_from_bbox(self.fig.bbox)
            self.restore_region(self._background)
            self._drawOverlays()
            self.blit(self.fig.bbox)

        elif dirtyFlag:  # Need full redraw
            self.draw()

        # Workaround issue of rendering overlays with some matplotlib versions
        if (_parse_version('1.5') <= self._matplotlibVersion < _parse_version('2.1') and
                not hasattr(self, '_firstReplot')):
            self._firstReplot = False
            if self._overlays or self._graphCursor:
                qt.QTimer.singleShot(0, self.draw)  # Request async draw

    # cursor

    _QT_CURSORS = {
        None: qt.Qt.ArrowCursor,
        BackendBase.CURSOR_DEFAULT: qt.Qt.ArrowCursor,
        BackendBase.CURSOR_POINTING: qt.Qt.PointingHandCursor,
        BackendBase.CURSOR_SIZE_HOR: qt.Qt.SizeHorCursor,
        BackendBase.CURSOR_SIZE_VER: qt.Qt.SizeVerCursor,
        BackendBase.CURSOR_SIZE_ALL: qt.Qt.SizeAllCursor,
    }

    def setGraphCursorShape(self, cursor):
        cursor = self._QT_CURSORS[cursor]

        FigureCanvasQTAgg.setCursor(self, qt.QCursor(cursor))
